{-# LANGUAGE PatternGuards, TupleSections #-}

module TypeChecker where

import Control.Monad
import Control.Monad.Identity
import Control.Monad.State
import Control.Monad.Error
import Control.Monad.Reader
import Control.Applicative -- ((<$>))

import Data.Map (Map)
import qualified Data.Map as Map
import qualified Data.Traversable as Traversable

import Debug.Trace

import Text.PrettyPrint

import Util

import Abstract
import Polarity
import Value
import TCM
import Eval
import SPos (nocc)
-- import CallStack
import TraceError

import Warshall -- size constraint checking

import Termination

-- import Completness




traceCheck msg a = a -- trace msg a 
traceCheckM msg = return () -- traceM msg
{-
traceCheck msg a = trace msg a 
traceCheckM msg = traceM msg
-}

  
{- DEAD CODE
runWhnf :: Signature -> TypeCheck a -> IO (Either TraceError (a,Signature))
runWhnf sig tc = (runErrorT (runStateT tc  sig)) 
-} 

doNf sig e = runErrorT (runReaderT (runStateT (whnf emptyEnv e >>= reify) (initWithSig sig)) emptyContext)
doWhnf sig e = runErrorT (runReaderT (runStateT (whnf emptyEnv e >>= whnfClos) (initWithSig sig)) emptyContext)


-- top-level functions -------------------------------------------

runTypeCheck :: TCState -> TypeCheck a -> IO (Either TraceError (a,TCState))
runTypeCheck st tc = runErrorT (runReaderT (runStateT tc st) emptyContext) 
-- runTypeCheck st tc = runCallStackT (runReaderT (runStateT tc st) emptyContext) []
 
typeCheck dl = runTypeCheck initSt (typeCheckDecls dl)

-- checking top-level declarations -------------------------------

echo :: MonadIO m => String -> m ()
echo = liftIO . putStrLn

echoTySig :: MonadIO m => Name -> Expr -> m ()
echoTySig n t = echo $ "I> " ++ n ++ " : " ++ show t

echoEPrefix = "E> "

echoTySigE :: MonadIO m => Name -> Expr -> m ()
echoTySigE n t = echo $ echoEPrefix ++ n ++ " : " ++ show t

echoDefE :: MonadIO m => Name -> Expr -> m ()
echoDefE n t = echo $ echoEPrefix ++ n ++ " = " ++ show t

-- the type checker returns pruned (extracted) terms 
-- with irrelevant subterms replaced by Irr
typeCheckDecls :: [Declaration] -> TypeCheck [EDeclaration]
typeCheckDecls []     = return []
typeCheckDecls (d:ds) = do 
  de  <- typeCheckDeclaration d                         
  dse <- typeCheckDecls ds                              
  return (de ++ dse)           

-- since a data declaration generates destructor declarations
-- we need to return a list here
typeCheckDeclaration :: Declaration -> TypeCheck [EDeclaration]
typeCheckDeclaration (OverrideDecl Fail ds) = do
  st <- get
  r <- (typeCheckDecls ds >> return True) `catchError` 
        (\ s -> do liftIO $ putStrLn ("block fails as expected, error message:\n" ++ show s)
                   return False)
  if r then fail "unexpected success" else do
    put st
    return []

typeCheckDeclaration (OverrideDecl TrustMe ds) = 
  newAssertionHandling Warning $ typeCheckDecls ds

typeCheckDeclaration (OverrideDecl Impredicative ds) = 
  goImpredicative $ typeCheckDecls ds

typeCheckDeclaration (DataDecl n sz co pos0 tel t0 cs) = 
  typeCheckDataDecl n sz co pos0 tel t0 cs

typeCheckDeclaration (LetDecl bla (TypeSig n t) e ) = 
   (
    do echoTySig n t
       te <- checkType t 
       vt <- whnf' te
       -- getEnv >>= \ rho -> traceCheckM $ "checking let-body " ++ show e ++ " : " ++ show vt ++ " in environment " ++ show rho
       ee <- checkExpr e vt
       e' <- solveAndModify e emptyEnv  -- solve size constraints
       rho <- getEnv
       let v = mkClos rho ee -- delay whnf computation
       -- v  <- whnf' ee -- WAS: whnf' e'
       addSig n (LetSig vt v)    -- late (var -> expr) binding, but ok since no shadowing
--       addSig n (LetSig vt e')    -- late (var -> expr) binding, but ok since no shadowing
       echoTySigE n te
       echoDefE   n ee
       return [LetDecl bla (TypeSig n te) ee]
   ) `throwTrace` n

typeCheckDeclaration (MutualFunDecl False co funs) = 
  -- traceCheck ("type checking a function block") $
  do
    funse <- typeCheckFuns co funs
    return $ [MutualFunDecl False co funse]

typeCheckDeclaration (MutualFunDecl True co funs) = 
  -- traceCheck ("type checking a block of measured function") $
  do
    funse <- typeCheckMeasuredFuns co funs
    return $ [MutualFunDecl False co funse]

typeCheckDeclaration (MutualDecl measured ds) = do
  -- first check type signatures
  -- we add the typings into the context, not the signature
  tss <- typeCheckMutualSigs ds
  -- then check bodies
  -- we need to construct a positivity graph
  edss <- addTypeSigs (Dec False SPos) tss $ mapM typeCheckMutualBody ds  
  -- check positivity graph
  return $ concat edss

-- check signatures of a flattened mutual block
typeCheckMutualSigs :: [Declaration] -> TypeCheck [TySig TVal]
typeCheckMutualSigs [] = return []
typeCheckMutualSigs (d:ds) = do
  ts@ (TypeSig n tv) <- typeCheckMutualSig d
  new' n (Domain tv defaultDec) $ do
    tss <- typeCheckMutualSigs ds
    return $ ts : tss

typeCheckSignature :: TySig Type -> TypeCheck (TySig TVal)
typeCheckSignature (TypeSig n t) = do
  echoTySig n t
  te <- checkType t
  tv <- whnf' te
  return $ TypeSig n tv

typeCheckMutualSig :: Declaration -> TypeCheck (TySig TVal)
typeCheckMutualSig (LetDecl ev ts e) =  
  typeCheckSignature ts
typeCheckMutualSig (DataDecl n sz co pos tel t cs) = 
  typeCheckSignature (TypeSig n (teleToType tel t))
typeCheckMutualSig (FunDecl co (ts, (ar, cls))) =
  typeCheckSignature ts 
typeCheckMutualSig (OverrideDecl TrustMe [d]) = 
  newAssertionHandling Warning $ typeCheckMutualSig d
typeCheckMutualSig (OverrideDecl Impredicative [d]) =
  goImpredicative $ typeCheckMutualSig d
typeCheckMutualSig d = fail $ "typeCheckMutualSig: panic: unexpected declaration " ++ show d 

typeCheckMutualBody :: Declaration -> TypeCheck [EDeclaration]
typeCheckMutualBody (DataDecl n sz co pos tel t cs) = do
  -- set name of mutual thing whose body we are checking
  checkingMutual n $
    -- 
    typeCheckDataDecl n sz co pos tel t cs

typeCheckDataDecl :: Name -> Sized -> Co -> [Pos] -> Telescope -> Type -> [Constructor] -> TypeCheck [EDeclaration]
typeCheckDataDecl n sz co pos0 tel t0 cs = enter n $
 (do -- sig <- gets signature
     let params = length tel
     -- in case we are dealing with a sized type, check that 
     -- the polarity annotation (if present) at the size arg. is correct.
     (p', pos, t) <- do
       case sz of
         Sized    -> do
           let polsz = if co==Ind then Pos else Neg
           t <- case t0 of
             Pi (TBind dec x (Sort Size)) b ->
               case (polarity dec) of
                 -- insert correct polarity annotation if none was there
                 Mixed -> return $ Pi (TBind (dec { polarity = polsz }) x (Sort Size)) b
                 pol | pol == polsz -> return t0
                 pol -> fail $ "sized type " ++ n ++ " has wrong polarity annotation " ++ show pol ++ " at Size argument, it should be " ++ show polsz
             t0 -> return t0
           return (params + 1, pos0 ++ [polsz], t)
         NotSized -> return (params, pos0, t0)
     -- compute full type signature (including parameter telescope)
     let dt = (teleToType tel t)
     echoTySig n dt
     {- mmh, this does not work,  e.g.  data Id (A : Set)(a : A) : A -> Set
        then A -> Set is not distinguishable from Set -> Set (GADT)
        unclear what to do...
     dte <- checkTele tel $ \ tele -> do
       te <- checkSmallType t
       return (teleToType tele te)
      -}
     -- get the target sort dsof the datatype
     (ds, dte) <- checkDataType p' dt -- TODO?: use above code?
     echoTySigE n dte
     v <- whnf emptyEnv dte
     let cis = map (analyzeConstructor co n tel) cs
     addSig n (DataSig { numPars = params 
                       , positivity = pos 
                       , isSized = sz
                       , isCo = co
                       , symbTyp = v
                       , constructors = cis
                       , etaExpand = False
                       , isTuple = False
-- if cs==[] then Just [] else Nothing
{- OLD CODE
                       , constructors = map namePart cs
                       -- at first, do not add destructors, get them out later
                       , destructors  = Nothing
                       , isFamily = t /= Set  -- currently UNUSED
 -}
                       })
     when (sz == Sized) $
           szType co params v

     (isRecList, cse) <- liftM unzip $ 
       mapM (typeCheckConstructor n dt sz co pos tel) cs
     -- check if patterns overlaps with any other
     let overlapList = zipWith (\ ci n -> any (overlaps (corePat ci)) $ map corePat $ take n cis ++ drop (n+1) cis) cis [0..] -- worst case quadratic, could be improved by exploiting symmetry

     -- find out which of the constructors are non-overlapping
     -- add destructors only for these
     let mkDestr fi = 
          case (fClass fi) of
             Field (Just (ty, arity, cl)) | not (erased $ fDec fi) && not (null $ fName fi) -> 
               [MutualFunDecl False Ind [(TypeSig (fName fi) ty,(arity,[cl]))]]  
             _ -> []
     -- cEtaExp = True means that all field names are present
     let mkDestrs ci False | cEtaExp ci 
            = concat $ map mkDestr (cFields ci)
         mkDestrs ci _ = []
     let decls = concat $ zipWith mkDestrs cis overlapList
     when (not (null decls)) $ 
        traceCheckM $ "generated destructors: " ++ show decls
     declse <- mapM (\ d@(MutualFunDecl False co [(TypeSig n t,(ar,cls))]) -> do 
                       echo $ "G> " ++ showFun co ++ " " ++ n ++ " : " ++ show t
                       echo $ "G> " ++ render (prettyFun n cls)
                       typeCheckDeclaration d) 
                 decls

     -- decide whether to eta-expand at this type
     -- all patterns need to be proper and non-overlapping
     -- at least one constructor needs to be eta-expandable
     let isPatIndFam = all (\ ci -> fst (cPatFam ci)) cis 
                    && not (or overlapList)
     -- do not eta-expand recursive constructors (might not terminate)
     let disableRec ci ov rec' = ci { cEtaExp  
           =  cEtaExp ci        -- all destructors present
           && fst (cPatFam ci)  -- proper pattern to compute indices
           && not ov            -- non-overlapping
           && not (co==Ind && rec') }         -- non-recursive
     let cis' = zipWith3 disableRec cis overlapList isRecList
     let typeEtaExpandable = isPatIndFam && (null cis || any cEtaExp cis') 
     traceEtaM $ "data " ++ n ++ " eta-expandable " ++ show typeEtaExpandable ++ " constructors " ++ show cis'
     modifySig n (\ dataSig -> 
                      dataSig { etaExpand = typeEtaExpandable
                              , constructors = cis'
                              , isTuple = length cis' == 1 && isPatIndFam
                              })
     -- compute extracted data decl
     let (tele, te) = typeToTele' (length tel) dte
     return $ (DataDecl n sz co pos tele te cse) : concat declse

   ) -- `throwTrace` n  -- in case of an error, add name n to the trace

{- OLD CODE
{- a record type is a data type that fulfills 3 conditions
   1. non-recursive
   2. exactly 1 constructor
   3. constructor carries names for each of its arguments
 -}
     case isPatIndFam params cs of
      Just [(c,ps)] -> do 
        when (t == Set && nonRec && length cs == 1) $ do
         let constrName   = namePart (head cs) 
         let (ctel, core) = typeToTele (typePart (head cs)) 
         let destrNames = map (\ (dec,x,t) -> x) ctel
         let isRec = all (/= "") destrNames
         when isRec $ do 
  --         echo $ "+++ generating record destructors +++"
  {- now, generate destructors
  
  data Sigma (A : Set) (B : A -> Set) : Set
  {
    pair : (fst : A) -> (snd : B fst) -> Sigma A B
  }
  fst : [A : Set] -> [B : A -> Set] -> (p : Sigma A B) -> A
  { fst A B (pair .A .B _fst _snd) = _fst }
  snd : [A : Set] -> [B : A -> Set] -> (p : Sigma A B) -> B (fst p)
  { snd A B (pair .A .B _fst _snd) = _snd }
   -}      
           -- choose a name for the record to destroy
           let recName = "record_of_type_" ++ n
           let parNames = map (\ (_,x,_) -> x) tel
           -- substitute variable "fst" by application "fst A B p"
           let phi x = if x `elem` destrNames 
                         then App (Def x) (map Var (parNames ++ [recName]))
                         else Var x
           let prefix x =  "destructor_argument_" ++ x
           let pattern = ConP co constrName (map (DotP . Var) parNames 
                                          ++ map (VarP . prefix) destrNames)
           let mkDestr (_,x,t) = MutualFunDecl Ind [(TypeSig x (teleToTypeErase tel t'),[clause])] where 
                   t' = (Pi notErased recName core . parSubst phi) t
                   clause = Clause (map VarP parNames ++ [pattern]) (Var (prefix x))
           let decls = map mkDestr ctel
           mapM_ typeCheckDeclaration decls
           modifySig n (\ dataSig -> dataSig { constructors = Just [(c,ps,destrNames)] })
         return ()
      _ -> return ()
   ) `throwTrace` n  -- in case of an error, add name n to the trace
END OLD CODE -}

{-
typeCheckConstructor :: Name -> Sized -> [Pos] -> Telescope -> Constructor -> TypeCheck ()
typeCheckConstructor d sz pos tel (TypeSig n t) = 
   ( 
   do sig <- gets signature 
      let tt = teleToTypeErase tel t
      echoTySig n tt
      let params = length tel 
      let p' = case sz of
                 Sized -> params + 1
                 NotSized -> params
      checkConType [] cxtEmpty p' tt
      let (_,target) = typeToTele tt
      checkTarget d tel target
      vt <- whnf  [] tt
      sposConstructor d 0 pos vt
      addSig n (ConSig vt)
      case sz of 
        Sized ->
            szConstructor d (length tel) vt
        NotSized -> return ()
      return ()
   ) `throwTrace` n
-}

-- returns True if constructor has recursive argument
typeCheckConstructor :: Name -> Type -> Sized -> Co -> [Pos] -> Telescope -> Constructor -> TypeCheck (Bool, EConstructor)
typeCheckConstructor d dt sz co pos tel (TypeSig n t) = 
   ( 
   do sig <- gets signature 
      let telE = map (mapDec (const (Dec True Mixed))) tel 
        -- parameters are erased and of mixed variants in types of constructors
      let tt = teleToType telE t
      echoTySig n tt
      let params = length tel 
{-
      let p' = case sz of
                 Sized -> params + 1
                 NotSized -> params
-}
      -- when checking constructor types,  do NOT resurrect telescope
      --   data T [A : Set] : Set { inn : A -> T A }
      -- should be rejected, since A ~= T A, and T A = T B means A ~=B for arb. A, B!
      -- add data name as spos var, to check positivity
      te <- addBinds (TBind (Dec False SPos) d dt:tel) $ 
        checkConType sz t -- do NOT resurrect telescope!!
      let tte = teleToType telE te -- DO resurrect here!
      let (argts,target) = typeToTele tt
      checkTarget d tel target
      vt <- whnf' tte
      {- old code was more accurate, since it evaluated before checking
         for recursive occurrence. 
      recOccs <- sposConstructor d 0 pos vt -- get recursive occurrences
      -}
      let recOccs = map (\ tb -> d `elem` usedDefs (boundType tb)) argts
      let isRec = foldl (||) False recOccs
      addSig n (ConSig params sz recOccs vt)
      when (sz == Sized) $ 
        szConstructor d co params vt -- check correct use of sizes
--      let (tele, te) = typeToTele (length tel) tte -- NOT NECESSARY
      echoTySigE n tte
      return (isRec, TypeSig n te)
   ) `throwTrace` n

{-
typeCheckFuns :: Co -> [(TypeSig,[Clause])] -> TypeCheck ()
typeCheckFuns co funs = do 
    mapM addFunSig funs
    mapM typeCheckFunSig funs
    -- type check and solve size constraints
    -- return clauses with meta vars resolved
    clss <- zipWithM typeCheckFunClauses [1..] funs 
    -- replace old clauses by new ones in funs
    let funs' = zipWith (\(tysig,cls) cls' -> (tysig,cl')) funs clss
    -- check sized types for admissibility
    mapM (admCheckFunSig (map (\ (TypeSig n t, cls) -> n) funs')) funs'
    liftIO $ terminationCheck funs'  
    mapM enableSig funs'      
    return ()
-}

typeCheckMeasuredFuns :: Co -> [Fun] -> TypeCheck [EFun]
typeCheckMeasuredFuns co funs = do 
    -- echo $ show funs
    fse <- mapM typeCheckFunSig funs -- also erases measure
    -- type check and solve size constraints
    -- return clauses with meta vars resolved
    (clss,clse) <- installFuns co funs $ liftM unzip $ zipWithM typeCheckFunClauses [1..] funs
    -- replace old clauses by new ones in funs
    let funs' = zipWith (\(tysig,(ar,cls)) cls' -> (tysig,(ar,cls'))) funs clss
    -- print reconstructed clauses
    mapM_ (\ (TypeSig n t, (ar, cls)) -> do
        echo $ "R> " ++ n ++ " : " ++ show t
        echo $ "R> " ++ (render $ prettyFun n cls))
      funs'
    -- get the list of mutually defined function names 
    let arities = map (\ (TypeSig n t, (ar, cls)) -> ar) funs
    -- replace in signature by erased clauses
    let funse = zip fse $ zip arities clse
    mapM (enableSig co) funse -- WAS : funs'      
    return $ funse

  where
    enableSig :: Co -> Fun -> TypeCheck ()
    enableSig co (TypeSig n t,(ar',cl')) = do
      vt <- whnf' t 
      addSig n (FunSig co vt ar' cl' True)
      -- add a let binding for external use
      v <- up False (vFun n) vt
      addSig (mkExtName n) (LetSig vt v)


typeCheckFuns :: Co -> [Fun] -> TypeCheck [EFun]
typeCheckFuns co funs = do 
    -- echo $ show funs
    fse <- mapM typeCheckFunSig funs
    mapM addFunSig funs
    -- type check and solve size constraints
    -- return clauses with meta vars resolved
    (clss,clse) <- setCo co $ liftM unzip $ zipWithM typeCheckFunClauses [1..] funs
    -- get the list of mutually defined function names 
    let names   = map (\ (TypeSig n t, (ar, cls)) -> n) funs
    let arities = map (\ (TypeSig n t, (ar, cls)) -> ar) funs
    -- check new clauses for admissibility, inserting "unusuable" flags in the patterns where necessary
    -- TODO: proper cleanup, proper removal of admissibility check!
    clss <- zipWithM (\ (tysig,_) cls' -> admCheckFunSig co names tysig cls') funs clss
    -- replace old clauses by new ones in funs
    let funs' = zipWith (\(tysig,(ar,cls)) cls' -> (tysig,(ar,cls'))) funs clss
    -- print reconstructed clauses
    mapM_ (\ (TypeSig n t, (ar, cls)) -> do
        echo $ "R> " ++ n ++ " : " ++ show t
        echo $ "R> " ++ (render $ prettyFun n cls))
      funs'
    terminationCheck funs'  
    -- replace in signature by erased clauses
    let funse = zip fse $ zip arities clse
    mapM enableSig funse -- WAS : funs'      
    return $ funse

  where

    addFunSig :: (TypeSig,(Int,[Clause])) -> TypeCheck ()
    addFunSig (TypeSig n t,(ar,cl)) = 
        do
          sig <- gets signature 
          vt <- whnf' t -- TODO: PROBLEM for internal extraction (would need te here)
          addSig n (FunSig co vt ar cl False) --not yet type checked / termination checked

    -- ADMCHECK FOR COFUN is not taking place in checking the lhs
    -- TODO: proper analysis for size patterns!
    -- admCheckFunSig mutualNames (TypeSig thisName thisType, clauses)
    admCheckFunSig :: Co -> [Name] -> TypeSig -> [Clause] -> TypeCheck [Clause] 
    admCheckFunSig CoInd  mutualNames (TypeSig n t) cls = return cls
    admCheckFunSig Ind    mutualNames (TypeSig n t) cls = -- traceCheck ("admCheckFunSig: checking admissibility of " ++ n ++ " : " ++ show t) $  
       (
        do -- a function is not recursive if did does not mention any of the
           -- mutually defined function names
           let usedNames = rhsDefs cls
           let notRecursive = all (\ n -> not (n `elem` usedNames)) mutualNames 
           -- for non-recursive functions, we can skip the admissibility check
           if notRecursive then 
              -- trace ("function " ++ n ++ " is not recursive") $
                return cls
            else -- trace ("function " ++ n ++ " is recursive ") $ 
              do vt <- whnf' t
                 admFunDef co cls vt
        ) `throwTrace` ("checking type of " ++ n ++ " for admissibility")
            

{-
    -- admCheckFunSig mutualNames (TypeSig thisName thisType, clauses)
    admCheckFunSig :: [Name] -> (TypeSig,[Clause]) -> TypeCheck () 
    admCheckFunSig mutualNames (TypeSig n t, cls) =   
       (
        do -- a function is not recursive if did does not mention any of the
           -- mutually defined function names
           let usedNames = rhsDefs cls
           let notRecursive = all (\ n -> not (n `elem` usedNames)) mutualNames 
           -- for non-recursive functions, we can skip the admissibility check
           when (not notRecursive) $ do
             vt <- whnf [] t
           when (not notRecursive) $ do
             vt <- whnf [] t              
             (case co of
                Ind -> do 
                   when (not (completeFun cls)) $
                      fail $ n ++ " : size pattern incomplete" 
                   let admpos = getSizeMatches cls -- admissible in which size variables?  
                   szCheckIndFun admpos vt `throwTrace` "type not admissible"

                CoInd -> do 
                   res <- admCoFun vt `throwTrace` "type not admissible"
                   case res of
                      CoFun -> return ()
                      SizedCoFun i ->
                         when (not (coFunSuccPat i cls)) $
                            fail $ n ++ " : corecursive size arguments can only be matched agains ($ i) for some size variable i"       
                )  -- TODO: this might be incorrect for mutual cofuns
        ) `throwTrace` ("type of " ++ n)
-}

    enableSig :: Fun -> TypeCheck ()
    enableSig (TypeSig n _,(ar',cl')) = do 
      (FunSig co vt ar cl _) <- lookupSymb n
      addSig n (FunSig co vt ar cl' True)
      -- add a let binding for external use
      v <- up False (vFun n) vt
      addSig (mkExtName n) (LetSig vt v)


-- typeCheckFunSig (TypeSig thisName thisType, clauses)
typeCheckFunSig :: (TypeSig,(Int,[Clause])) -> TypeCheck ETypeSig 
typeCheckFunSig (TypeSig n t, (ar, cls)) =   
   (
    do echoTySig n t
       te0 <- checkType t
       let te = eraseMeasure te0
       echoTySigE n te
       return $ TypeSig n te
    ) `throwTrace` ("type of " ++ n)

typeCheckFunClauses :: Int -> (TypeSig,(Int,[Clause])) -> TypeCheck ([Clause], [EClause])
typeCheckFunClauses k (TypeSig n t,(ar,cl)) =
   (do (cl',cle) <- checkFun t cl
       -- traceCheck (show (TypeSig n t)) $
        -- traceCheck (show cl') $
       echo $ render $ prettyFun n cle
       return (cl',cle)
   ) `throwTrace` n         

{-
checkConType :: Env -> SemCxt -> Int -> Expr -> TypeCheck ()
checkConType rho delta p e =
    case e of
      Pi dec x t1 t2 -> do
             case (len delta) < p of
               True ->
                   return ()
               False ->
                   checkSmallType rho delta t1
             v_t1 <- whnf rho t1
             let (k, delta') = cxtPush (Dec False) v_t1 delta
             checkConType (update rho x (VGen k)) (delta') p t2
      _ -> checkExpr rho delta e VSet
 -}

{-
-- check that arguments are stypes
-- check that result is a set 
--  ( params were already checked by checkDataType ) 
checkConType :: Int -> Expr -> TypeCheck Extr
checkConType p e = 
    case e of
      Pi dec x t1 t2 -> do
             l <- getLen
             when (l >= p) $ checkSmallType t1
             addBind x (Dec False) t1 $ do
               checkConType p t2 
      _ -> checkSmallType e 

-- checkConType numpars t = te
-- check that arguments are stypes
-- check that result is a set 
--  ( params were already checked by checkDataType ) 
-- called initially in the empty context
-- the extracted type is does not include the parameter telescope!
checkConType :: Sized -> Int -> Expr -> TypeCheck Extr
checkConType NotSized 0 t = checkConTypeCore t 
-- checkConType Sized 0 t = checkConTypeCore t 
checkConType Sized 0 t =
    case t of
      Pi dec x (Sort Size) t2 -> do
             addBind x (Dec False) (Sort Size) $ do
               t2e <- checkConTypeCore t2
               return $ Pi dec x (Sort Size) t2e 
      _ -> fail $ "checkConType: internal error, t=" ++ show t

checkConType sz n t =
    case t of
      Pi dec x t1 t2 -> do
             -- no need to check parameter t1, already checked in data decl
             addBind x (Dec False) t1 $ do
               checkConType sz (n-1) t2 
      _ -> fail $ "checkConType: internal error, n=" ++ show n ++ " t=" ++ show t
-}

-- checkConType sz t = te
-- check that arguments are stypes
-- check that result is a set 
--  ( params were already checked by checkDataType and are not included in t ) 
-- called initially in the context consisting of the parameter telescope
checkConType :: Sized -> Expr -> TypeCheck Extr
checkConType NotSized t = checkConTypeCore t 
checkConType Sized t =
    case t of
      Pi tb@(TBind dec x (Sort Size)) t2 -> do
             addBind (mapDec resurrectDec tb) $ do
               t2e <- checkConTypeCore t2
               return $ Pi tb t2e 
      _ -> fail $ "checkConType: internal error, t=" ++ show t

checkConTypeCore :: Expr -> TypeCheck Extr
checkConTypeCore t = do
  (s, te) <- checkingCon True $ inferType t
  case s of
    Set{} -> return te 
    CoSet{} -> return te
    _ -> fail $ "checkConType: type " ++ show t ++ " of constructor not a set"
{-
    case t of
      Pi dec x t1 t2 -> do
             t1e <- checkSmallType t1
             addBind x (Dec False) t1 $ do
               t2e <- checkConTypeCore t2
               return $ Pi dec x t1e t2e 
      _ -> checkSmallType t 
 -}       

-- check that the data type and the parameter arguments (written down like declared in telescope) 
checkTarget :: Name -> Telescope -> Type -> TypeCheck()
checkTarget d tel tg =  
    case spineView tg of
      (Def (DefId Dat n), args) | n == d -> checkParams tel (take (length tel) args)
      _ -> throwErrorMsg $ "target mismatch"  ++ show tg

    where checkParams :: Telescope -> [Expr] -> TypeCheck ()
          checkParams [] [] = return ()
          checkParams (tb : tl) ((Var n') : el) | boundName tb == n' 
            = checkParams tl el
          checkParams tl al = throwErrorMsg $ "target param mismatch " ++ 
            d ++ " " ++ show tel ++ " != " ++ show tg ++ "\ncheckParams " ++ show tl ++ " " ++ show al ++ " failed"

                      
-- check that params are types
-- check that arguments are stypes
-- check that target is set
checkDataType :: Int -> Expr -> TypeCheck (Sort Expr, Extr)
checkDataType p e = do
  traceCheckM ("checkDataType " ++ show e ++ " p=" ++ show p)
  case e of
     Pi tb@(TBind dec x t1) t2 -> do 
             k <- getLen
             traceCheckM ("length of context = " ++ show k)  
             -- t1e <- checkingDom $ if k <= p then checkType t1 else checkSmallType t1    
             (s1, t1e) <- checkingDom $ inferType t1
             addBind (TBind defaultDec x t1) $ do
               (s, t2e) <- checkDataType p t2
               -- when k <= p $ ltSort s1 s -- check size of indices (disabled)
               return $ (s, Pi (TBind dec x t1e) t2e)  
     Sort s@(Set{})   -> return (s, e)
     Sort s@(CoSet{}) -> return (s, e)
     _ -> throwErrorMsg "doesn't target Set or CoSet"
                      

{- Kind inference

          i    : Size              : Type
      t : Nat  : Set  : Set1 : ... : Type = Set\omega
  p : P : Prop : Set  : ...

Functional, cumulative PTS (s,s',s') written (s,s')

  (Size,s)   s != Size    size-dependency
  (s,Prop)                impredicative Prop
  (Set_i,Set_j)  i <= j   predicativity

Kind    can be used to construct Kinds
term t  terms, types, universes, proofs, propositions
type T  types, universes, propositions
size i  types, universes, propositions
prf  p  proofs
pred P  types, universes, propositions

We like to infer kinds of expressions

  Tm < Set < Set1 < Set2 < ...

For  t : A  if kind(A) = Tm then t is a term, 
                       = Set then t is a type,
                       = Set1 then t is a type1 (e.g, a universe) ...

Then,  if  t : (x : A) -> B
      and  kind(A) `irrelevantFor` kind(B)   [ with irrelevantFor := > ]

we can change the type signature to

           t : [x : A] -> B

This is because you cannot eliminate a type to produce a term.

  kind(Set)  = Set
  kind(Size) = Size -- this means that we treat sizes as types, they cannot
  kind(s)    = s    -- if s is a sort
  kind((x : A) -> B) = kind(B)
  kind(A : Set0) = Tm
  kind(A : Prop) = Prf
  kind(A : Size) = <<impossible>>
  kind(A : Setk) = k-1

irrFor Tm  _    = False
irrFor Ty Tm    = True
irrFor Ty Prf   = True
irrFor Ty _     = False
irrFor Size Tm  = True
irrFor Size Prf = True

With Prop, one can 
-}

    
-- type checking

{-
-- entry point 1: possibly resurrects
checkExpr :: Expr -> TVal -> TypeCheck Extr
checkExpr e v = maybeResurrect v $ checkExpr' e v 

maybeResurrect :: TVal -> TypeCheck a -> TypeCheck a
maybeResurrect v = if v == VSet || v == VSize then resurrect else id

-- entry point 2: possibly resurrects
checkExprDec :: Dec -> Expr -> TVal -> TypeCheck Extr
checkExprDec dec e v = do ee <- maybeResurrectDec dec v $ checkExpr' e v  -- ignore erasure
                          return $ ee -- if erased dec then Irr else ee

-- superseded by TCM.applyDec                           
maybeResurrectDec :: Dec -> TVal -> TypeCheck a -> TypeCheck a
maybeResurrectDec dec v = if erased dec || v == VSet || v == VSize then resurrect else id
-}
{-
-- entry point 2: possibly resurrects, adjusts polarities
checkExprDec :: Dec -> Expr -> TVal -> TypeCheck Extr
checkExprDec dec e v = do 
  ee <- maybeResurrectDec dec $ checkExpr' e v  -- ignore erasure
  return $ ee -- if erased dec then Irr else ee
-}

-- entry point 1: redundant
checkExpr :: Expr -> TVal -> TypeCheck Extr
checkExpr e v = checkExpr' e v 

{-
-- checkExpr e tv = Just e' | Nothing
-- instead of e' = Irr return Nothing
-}

-- checkExpr e tv = e'
-- where e' is the pruned version of e
--
-- this is *internal* extraction in the style of Barras and Bernardo
-- e.g., does not prune t : Id A a b
-- thus, we can use the pruned version for evaluation!
checkExpr' :: Expr -> TVal -> TypeCheck Extr
checkExpr' e v = do
  l <- getLen
  enter ("checkExpr' " ++ show l ++ " |- " ++ show e ++ " : " ++ show v) $ do

   ce <- ask
   traceCheck ("checkExpr': " ++ show (renaming ce) ++ ";" ++ show (context ce) ++ " |- " ++ show e ++ " : " ++ show v ++ " in env" ++ show (environ ce)) $ do
  
    (case (e,v) of

{- In the presence of full bracket types, 
   we could implement the following "resurrecting version of let"

   Gamma |- s : [A]   
   Gamma, x:A |- t : C   Gamma, x:A, y:A |- t = t[y/x] : C
   -------------------------------------------------------
   Gamma |- let x:[A] = s in t : C

 -}

      (LLet (TBind dec x t1) e1 e2,_) ->
          do 
            t1e <- checkType t1
            v_t1 <- whnf' t1
            e1e <- applyDec dec $ checkExpr' e1 v_t1
            v_e1 <- whnf' e1
            new x (Domain v_t1 dec) $ \ vx -> do
              addRewrite (Rewrite vx v_e1) [v] $ \ [v'] -> do
                e2e <- checkExpr' e2 v'
                return $ LLet (TBind dec x t1e) e1e e2e  -- if e2e==Irr then Irr else LLet n t1e e1e e2e  
-- Dependent let: not checkable in rho;Delta style
--            v_e1 <- whnf rho e1
--            checkExpr' (update rho n v_e1) (v_t1 : delta) e2 v  

      (Case (Var x) [Clause [SuccP (VarP y)] (Just rhs)], v) -> do
          (tv,ex) <- resurrect $ inferExpr (Var x)
          subtype tv (VSort Size)
          vx@(VGen i) <- whnf' (Var x)
          endsInSizedCo i v
          newWithGen y (Domain (VSort Size) defaultDec) $ \ j vy -> do
            let vp = VSucc vy
            addSizeRel j i $
              addRewrite (Rewrite vx vp) [v] $ \ [v'] -> do
                rhse <- checkRHS [] rhs v'
                return (Case (Var x) [Clause [SuccP (VarP y)] (Just rhse)])

      (Case e cs, _) -> do
          (tv,ee) <- inferExpr e
          ve <- whnf' ee
          -- tv' <- sing' ee tv -- DOES NOT WORK
          cle <- checkCases ve (arrow tv v) cs
          return $ Case ee cle

      (e,v) | inferable e -> do 
          (v2,ee) <- inferExpr e
          checkSubtype ee v2 v 
          return ee

      _ -> checkForced e v

      ) -- >> (trace ("checkExpr successful: " ++ show e ++ ":" ++ show v) $ return ())

-- check expression after forcing the type
checkForced :: Expr -> TVal -> TypeCheck Expr
checkForced e v =  do
  l <- getLen
  v <- force v
  enter ("checkForced " ++ show l ++ " |- " ++ show e ++ " : " ++ show v) $ do
    case (e,v) of

      -- (_, VBound{}) -> no intro for constrained types, but inference

      -- metavariables must have type size
      (Meta i, VSort Size) -> do 
         addMeta l i
         return $ Meta i -- Irr -- sizes are irrelevant in terms, Meta's cannot be in types

{- Following Awodey/Bauer 2001, the following rule is valid

   Gamma, x:A |- t : B    Gamma, x:A, y:A |- t = t[y/x] : B 
   --------------------------------------------------------
   Gamma |- \xt : Pi x:[A]. B

TODO: add bracket types to MiniAgda
-}
{-
QUESTIONABLE: this rule is used for polymorphism, and it may not be sound
      (Lam _ y e1, VPi dec x va env t1) -> do
          rho <- getEnv  -- get the environment corresponding to Gamma
          new y (Domain va (resurrectDec dec)) $ \ vy -> do
            v_t1 <- whnf (update env x vy) t1
            -- traceCheckM $ "checking " ++ show e1 ++ " : " ++ show v_t1 
            e1e <- checkExpr' e1 v_t1  
            when (erased dec) $ do  -- now check invariance of the e1 
              new y (Domain va (resurrectDec dec)) $ \ vy' -> do
                ve  <- whnf (update rho y vy)  e1e 
                ve' <- whnf (update rho y vy') e1e
                eqVal v_t1 ve ve'  -- BUT: ve' does not have type v_t1 !? 
            -- prune the lambda if body has been pruned
            return $ if e1e==Irr then Irr else Lam y e1e
 -}   
 
-- NOW just my rule (LICS 2010 draft) a la Barras/Bernardo

      (Lam _ y e1, VPi dec x va env t1) -> do
          rho <- getEnv  -- get the environment corresponding to Gamma
          new y (Domain va dec) $ \ vy -> do
            v_t1 <- whnf (update env x vy) t1
            -- traceCheckM $ "checking " ++ show e1 ++ " : " ++ show v_t1 
            e1e <- checkExpr' e1 v_t1  
            return $ Lam dec y e1e

{- 
      (Pi dec n t1 t2, VSort Set) ->
          do t1e <- if t1 == Sort Size then return t1 
                     else checkExpr' t1 $ VSort Set  -- Size is not a set TODO:proper PTS rules!
             addBind n (Dec False) t1e $ do  -- ignore erasure flag in Pi!
               t2e <- checkExpr' t2 $ VSort Set
             -- never prune a Pi
               return $ Pi dec n t1e t2e
 
      (Pi dec n t1 t2, VSort (CoSet vi)) ->
          do t1e <- if t1 == Sort Size then return t1 
                     else checkExpr' t1 $ VSort Set  -- Size is not a set TODO:proper PTS rules!
             addBind n (Dec False) t1e $ do  -- ignore erasure flag in Pi!
               t2e <- checkExpr' t2 $ VSort $ CoSet $ maybe vi id $ predSize vi
             -- never prune a Pi
               return $ Pi dec n t1e t2e
-}
 
      (Max es, VSort Size) -> do
              ese <- mapM (\ e -> checkExpr' e $ VSort Size) es
              return $ Max ese

      (Succ e2, VSort Size) -> do
              e2e <- checkExpr' e2 $ VSort Size
              return $ Succ e2e
{-
              -- prune sizes
              return $ if e2e==Irr then Irr else Succ e2e
-}
      _ -> do (v2,ee) <- inferExpr e
              checkSubtype ee v2 v 
              return ee

-- checkSubtype  expr : infered_type <= ascribed_type
checkSubtype :: Expr -> TVal -> TVal -> TypeCheck ()
checkSubtype e v2 v = do
    rho <- getEnv
    traceCheckM $ "computing singleton <" ++ show e ++ " : " ++ show v2 ++ "> in environment " ++ show rho
    v2principal <- sing rho e v2
    traceCheckM $ "subtype checking " ++ show v2principal ++ " ?<= " ++ show v ++ " in environment " ++ show rho
    subtype v2principal v


-- ptsRule s1 s2 = s  if (s1,s2,s) is a valid rule
-- precondition: s1,s2 are proper sorts, i.e., not Size or Tm
ptsRule :: Sort Val -> Sort Val -> TypeCheck (Sort Val)
ptsRule s1 s2 = do
  cxt <- ask
  let parametric = checkingConType cxt  -- are we dealing with a parametric pi?
  let err = "ptsRule " ++ show (s1,s2) ++ " " ++ (if parametric then "(in type of constructor)" else "") ++ ": "
  case (s1,s2) of
    (Type,_) -> fail $ err ++ "domain too big"
    (Set v1, Set v2) -> 
      if parametric then do 
         leqSize Pos v1 v2 -- when we are checking a constructor, to reject 
         {- data Bad (A : Set) : Set { bad : (A : Set) -> Bad A } -}
         return s2
       else return $ Set $ maxSize [v1,v2] 
    (CoSet v1, Set VZero) -> 
      if parametric then return $ CoSet v1
       else if v1==VInfty then return $ Set VZero
             else fail $ err ++ "domain cannot be sized"
    (CoSet v1, CoSet v2) -> if parametric then do 
                              let v2' = maybe v2 id $ predSize v2
                              case minSize v1 v2 of
                                Just v -> return $ CoSet v
                                Nothing -> fail $ err ++ "min" ++ show (v1,v2) ++ " does not exist"
                            else if v1==VInfty then return $ CoSet $succSize v2 
                                  else fail $ err ++ "domain cannot be sized"
    _ -> return s2
    

-- inferType t = (s, te)
inferType :: Expr -> TypeCheck (Sort Val, Extr)
inferType t = do
  (sv, te) <- inferExpr t
  case sv of
    VSort s | not (s `elem` [Tm,Size]) -> return (s,te) 
    _ -> fail $ "inferExpr: expected " ++ show t ++ " to be a type!"
  
-- inferExpr e = (tv, s, ee)
-- input : expr e | inferable e
-- output: type tv, kind s, and erased form ee of e
-- the kind tells whether e is a term, a size, a set, ...
inferExpr :: Expr -> TypeCheck (TVal, Extr) 
inferExpr e = do
  (tv, ee) <- inferExpr' e
  case tv of
    VGuard beta vb -> do
      checkGuard beta
      return (vb, ee)
    _ -> return (tv, ee)

checkGuard :: Bound Val -> TypeCheck ()
checkGuard beta@(Bound mu mu') = enterTrace ("checkGuard " ++ show beta) $
  ltMeasure mu mu'

-- inferExpr' might return a VGuard, this is removed in inferExpr
inferExpr' :: Expr -> TypeCheck (TVal, Extr) 
inferExpr' e = enter ("inferExpr' " ++ show e) $
  let returnSing ee tv = do tv' <- sing' ee tv
                            return (tv', ee)
  in 
    (case e of

      Var x -> do 
        -- traceCheckM ("infer variable " ++ x) $ 
        item <- lookupName1 x
        let err = "inferExpr: variable " ++ x ++ " : " ++ show (typ item) ++ 
                  " may not occur"
        let dec = decor item
        let pol = polarity dec
        if erased dec then 
          fail $ err ++ ", because it is marked as erased"
         else if not (leqPol pol SPos) then
          fail $ err ++ ", because it has polarity " ++ show pol
         else do 
           -- traceCheckM ("infer variable " ++ x ++ " : " ++ show  (typ item))
           return $ (typ item, Var x) -- TODO: (typ item, kind item, Var x)

      Sort (CoSet e) -> do
        ee <- checkExpr e (VSort Size)
        return (VSort (Set (VSucc VZero)), Sort $ CoSet ee)
      Sort (Set e) ->  do
        ee <- checkExpr e (VSort Size)
        v  <- whnf' e
        return (VSort (Set (succSize v)), Sort $ Set ee)
      Sort Size -> return (VSort TSize, e)
      Zero -> return (VSort Size, Zero)
      Infty -> return (VSort Size, Infty)

      Pi (TBind dec n t1) t2 -> do
        -- make sure that in a constructor declaration the constructor args are
        -- mixed-variant (there is no subtyping between constrs anyway)
        when (polarity dec /= Mixed) $ do
          checkCon <- asks checkingConType
          when checkCon $ 
            fail $ "constructor arguments must be declared mixed-variant"

        (s1, t1e) <- checkingDom $ checkingCon False $ inferType t1 -- switch off parametric Pi
        addBind (TBind defaultDec n t1e) $ do -- ignore erasure flag AND polarity in Pi!
          (s2, t2e) <- inferType t2
          ce <- ask
          s <- if impredicative ce && erased dec && s2 == Set VZero then return s2 else ptsRule s1 s2 -- Impredicativity!
          -- s <- ptsRule s1 s2
          return (VSort s, Pi (TBind dec n t1e) t2e)

      Pi (TMeasure (Measure mu)) t2 -> do
        mue <- mapM (flip checkExpr (VSort Size)) mu
        (s, t2e) <- inferType t2
        return (VSort s, Pi (TMeasure (Measure mue)) t2e)

      Pi (TBound (Bound (Measure mu) (Measure mu'))) t2 -> do
        mue  <- mapM (flip checkExpr (VSort Size)) mu
        mue' <- mapM (flip checkExpr (VSort Size)) mu'
        (s, t2e) <- inferType t2
        return (VSort s, Pi (TBound (Bound (Measure mue) (Measure mue'))) t2e)

      Sing e1 t -> do
        (s,te) <- inferType t
        tv <- whnf' te
        e1e <- checkExpr e1 tv
        return (VSort $ s, Sing e1e te)
--      App e1 [e2] ->
      App e1 e2 -> checkingCon False $ 
          do
            (v, e1e) <- inferExpr e1
            v <- force v -- if v is a corecursively defined type in Set, unfold!
            case v of
               VPi dec x av env b -> do
                  (v2,e2e) <- 
                    if inferable e2 then do
                  -- if e2 has a singleton type, we should not take v2 = whnf e2
                  -- but use the single value of e2
                  -- this is against the spirit of bidir. checking
                           -- if checking a type we need to resurrect
                           (av', e2e) <- applyDec dec $ inferExpr e2
                           case av' of
                              VSing v2 av'' -> do subtype av' av
                                                  return (v2,e2e)
                              _ -> do checkSubtype e2e av' av
                                      v2 <- whnf' e2e 
                                      return (v2, e2e)
                         else do
                           e2e <- applyDec dec $ checkExpr e2 av
                           v2 <- whnf' e2
                           return (v2, e2e)
                  bv <- whnf (update env x v2) b
                  return (bv, App e1e (if erased dec then Erased e2e else e2e))
-- if e1e==Irr then Irr else if e2e==Irr then e1e else App e1e [e2e]) 
               _ -> throwErrorMsg $ "inferExpr : expected Pi with expression : " ++ show e1 ++ "," ++ show v
 
--      App e1 (e2:el) -> inferExpr $ (e1 `App` [e2]) `App` el  
      (Def id) -> do -- traceCheckM ("infer defined head " ++ n) 
         mitem <- errorToMaybe $ lookupName1 (name id)
         case mitem of -- first check if it is also a var name
           Just item -> -- we are inside a data decl checking constructors
             if not $ leqPol (polarity $ decor item) SPos then
               fail $ "recursive occurrence of " ++ name id ++ " not strictly positive"
             else return (typ item, e) 
           Nothing -> -- otherwise, it is not the data type name just being defined
                 do sige <- lookupSymb (name id)
                    case sige of
                      (DataSig { symbTyp = tv }) -> return (tv, e)
                      (FunSig  { symbTyp = tv }) -> return (tv, e)
                      (ConSig  { symbTyp = tv }) -> returnSing e tv  -- constructors have sing.type!
                      (LetSig tv v) -> return (tv, e) -- return $ vSing v tv 
{-
      (Con _ n) -> do sig <- gets signature
                      case (lookupSig n sig) of
      (Let n) -> do sig <- gets signature 
                    case (lookupSig n sig) of
-}
      _ -> throwErrorMsg $ "cannot infer type of " ++ show e
     ) >>= \ tv -> ask >>= \ ce -> 
         traceCheck ("inferExpr: " ++ show (renaming ce) ++ ";" ++ show (context ce) ++ " |- " ++ show e ++ " :=> " ++ show tv ++ " in env" ++ show (environ ce)) $
--         traceCheck ("inferExpr: " ++ show e ++ " :=> " ++ show tv) $ 
           return tv


{- BAD IDEA!
improveDec :: Dec -> TVal -> Dec
improveDec dec v = if v == VSet || v == VSize then erased else dec
-}

{-
-- entry point 3: resurrects
checkType :: Expr -> TypeCheck Extr
checkType e = (resurrect $ checkType' e) `throwTrace` ("not a type: " ++ show e )

checkType' :: Expr -> TypeCheck Extr
checkType' e = case e of
    Sort s -> return e
    Pi dec x t1 t2 -> do
        t1e <- checkType' t1
        -- ignore erasure flag in types!
--        t1v <- whnf' t1e
--        new' x (Domain (Dec False) t1v) $ do
        addBind x (Dec False) t1e $ do
          t2e <- checkType' t2
          return $ Pi dec x t1e t2e  -- Pi (improveDec dec t1v) x t1e t2e  
    _ -> checkExpr' e $ VSort Set
-}

checkType :: Expr -> TypeCheck Extr
checkType t = 
  enter ("not a type: " ++ show t) $
    resurrect $ do
      (s, te) <- inferType t
      leqSort Pos s Type
      return te

checkSmallType :: Expr -> TypeCheck Extr
checkSmallType t = 
  enter ("not a set: " ++ show t) $
    resurrect $ do
      (s, te) <- inferType t
      case s of
        Set VZero -> return te
        CoSet{} -> return te
        _ -> fail $ "expected " ++ show s ++ " to be Set or CoSet _"

{-
-- small type
checkSmallType :: Expr -> TypeCheck Extr
checkSmallType e  = (resurrect $ checkExpr' e $ VSort Set) `throwTrace` ("not a set: " ++ show e )
-}

-- check telescope and add bindings to contexts
checkTele :: Telescope -> (ETelescope -> TypeCheck a) -> TypeCheck a
checkTele = checkTele' []

checkTele' :: ETelescope -> Telescope -> (ETelescope -> TypeCheck a) -> TypeCheck a
checkTele' acc [] k = k (reverse acc)
checkTele' acc (tb@(TBind dec x t):tel) k = do
  te <- checkType t
  addBind tb $
    checkTele' (TBind dec x te:acc) tel k
 
-- the integer argument is the number of the clause, used just for user feedback
checkCases :: Val -> TVal -> [Clause] -> TypeCheck [EClause]
checkCases = checkCases' 1

checkCases' :: Int -> Val -> TVal -> [Clause] -> TypeCheck [EClause]
checkCases' i v tv [] = return []
checkCases' i v tv (c:cl) = do 
    ce  <- checkCase i v tv c
    cle <- checkCases' (i + 1) v tv cl
    return $ ce : cle

checkCase :: Int -> Val -> TVal -> Clause -> TypeCheck EClause
checkCase i v tv cl@(Clause [p] mrhs) =
  -- traceCheck ("checking case " ++ show i) $
    (
    do   
      (flex,ins,cxt,vt,pe,absp) <- checkPattern False [] [] tv p
      local (\ _ -> cxt) $ do
        mapM (checkGoal ins) flex
        case (absp,mrhs) of
           (True,Nothing) -> return (Clause [pe] Nothing) 
           (False,Nothing) -> fail ("missing right hand side in case " ++ showCase cl)
           (True,Just rhs) -> fail ("absurd pattern requires no right hand side in case " ++ showCase cl)
           (False,Just rhs) -> do
              vp <- whnf' (patternToExpr p)
      --        vp <- patternToVal p -- BUG: INTRODUCES FRESH GENS, BUT THEY HAVE ALREADY BEEN INTRODUCED IN checkattern
              addRewrite (Rewrite v vp) [vt] $ \ [vt'] -> do
                rhse <- checkRHS ins rhs vt'
                return (Clause [pe] (Just rhse))
                -- rhs' <- solveAndModify rhs (environ cxt)
                -- return (Clause [p] rhs')
    ) `throwTrace` ("case " ++ show i)

-- type check a function

checkFun :: Type -> [Clause] -> TypeCheck ([Clause],[EClause])
checkFun t cl = do
  tv <- whnf' t
  checkClauses tv cl
 
-- the integer argument is the number of the clause, used just for user feedback
checkClauses :: TVal -> [Clause] -> TypeCheck ([Clause],[EClause])
checkClauses = checkClauses' 1

checkClauses' :: Int -> TVal -> [Clause] -> TypeCheck ([Clause],[EClause])
checkClauses' i tv [] = return ([],[])
checkClauses' i tv (c:cl) = do 
    (c',ce)   <- checkClause i tv c
    (cl',cle) <- checkClauses' (i + 1) tv cl
    return $ (c' : cl', ce : cle)

-- checkClause i tv cl = (cl', cle)
-- checking one equation cl of a function at type tv
-- solve size constraints
-- substitute solution into clause, resulting in cl'
-- return also extracted clause cle
checkClause :: Int -> TVal -> Clause -> TypeCheck (Clause, EClause)
checkClause i v cl@(Clause pl mrhs) =
  -- traceCheck ("checking function clause " ++ show i) $
    (
    do   
      (flex,ins,cxt,vt,ple,absp) <- checkPatterns False [] [] v pl
      local (\ _ -> cxt) $ do
        mapM (checkGoal ins) flex
        -- TODO: insert meta var solution in dot patterns
        case (absp,mrhs) of
           (True,Nothing) -> return (cl, Clause ple Nothing) 
           (False,Nothing) -> fail ("missing right hand side in clause " ++ show cl)
           (True,Just rhs) -> fail ("absurd pattern requires no right hand side in clause " ++ show cl)
           (False,Just rhs) -> do
              rhse <- checkRHS ins rhs vt
              env  <- getEnv
              rhs' <- solveAndModify rhs env
              return (Clause pl (Just rhs'), Clause ple (Just rhse))
    ) `throwTrace` ("clause " ++ show i)



type Substitution = [(Int,TVal)]

type DotFlex = (Int,(Expr,Domain))

-- left over goals
data Goal = DotFlex Int Expr Domain
          | MaxMatches Int TVal 

-- checkPatterns is initially called with an empty local context
-- in the type checking monad
checkPatterns :: Bool -> [Goal] -> Substitution -> TVal -> [Pattern] -> TypeCheck ([Goal],Substitution,TCContext,TVal,[EPattern],Bool)
checkPatterns erased flex ins v pl = 
  case v of
    VMeasured mu vb -> setMeasure mu $ checkPatterns erased flex ins vb pl
    VGuard beta vb -> fail $ "checkPattern at type " ++ show v ++ " --- introduction of constraints not supported"
    _ -> case pl of
      [] -> do cxt <- ask
               return (flex,ins,cxt,v,[],False)
      (p:pl') -> do (flex',ins',cxt',v',pe,absp) <- checkPattern erased flex ins v p 
                    local (\ _ -> cxt') $ do
                      (flex'',ins'',cxt'',v'',ple,absps) <- checkPatterns erased flex' ins' v' pl' 
                      return (flex'',ins'',cxt'',v'', pe:ple, absp || absps) -- if pe==IrrP then ple else pe:ple)
       
{- 
checkPattern erased flex subst v p = (flex', subst', cxt', v', pe)

Input : 
  erased: are we checking an erased argument? (constr. pat. needs to be forced!)
  flex  : list of pairs (flexible variable, its dot pattern + supposed type)
  subst : list of pairs (flexible variable, its valuation)
  cxt   : in monad, containing
    rho   : binding of variables to values
    delta : binding of generic values to their types
  v     : type of the expression \ p -> t
  p     : the pattern to check

Output
  v'    : type of t
  pe    : erased pattern
  absp  : did we encounter an absurd pattern
-}

checkPattern :: Bool -> [Goal] -> Substitution -> TVal -> Pattern -> TypeCheck ([Goal],Substitution,TCContext,TVal,EPattern,Bool)
checkPattern er flex ins v p = -- ask >>= \ TCContext { context = delta, environ = rho } -> trace ("checkPattern" ++ ("\n  dot pats: " +?+ show flex) ++ ("\n  substion: " +?+ show ins) ++ ("\n  environ : " +?+ show rho) ++ ("\n  context : " +?+ show delta) ++ "\n  pattern : " ++ show p ++ "\n  at type : " ++ show v ++ "\t<>") $ 
 (
 do 
  case v of
    VPi dec x av env b -> do
       let erased' = er || erased dec
       let decEr   = dec {erased = erased'}
       let maybeErase p = if erased' then ErasedP p else p
       case p of
          VarP y -> new y (Domain av decEr) $ \ xv -> do
                       bv <- whnf (update env x xv) b
                       cxt' <- ask 
                       return (flex, ins, cxt', bv, maybeErase $ VarP y, False)
          SizeP z y -> do
                    resurrect $ checkExpr (Var z) $ VSort Size
                    newWithGen y (Domain av decEr) $ \ j xv -> do
                       bv <- whnf (update env x xv) b
                       VGen k <- whnf' (Var z)
                       addSizeRel j k $ do
                         cxt' <- ask 
                         return (flex, ins, cxt', bv, maybeErase $ VarP y, False)

          AbsurdP -> do
                 when (isFunType av) $ fail ("absurd pattern " ++ show p ++ " does not match function types, like " ++ show av)     
                 bv   <- whnf (update env x VIrr) b
                 cxt' <- ask 
                 return (MaxMatches 0 av : flex, ins, cxt', bv, maybeErase $ AbsurdP, True)
{-
                 cenvs <- matchingConstructors av  -- TODO: av might be MVar
                                                   -- need to be postponed 
                 case cenvs of
                    [] -> do bv   <- whnf (update env x VIrr) b
                             cxt' <- ask 
                             return (flex, ins, cxt', bv, maybeErase $ AbsurdP, True)
                    _ -> throwErrorMsg $ "type " ++ show av ++ " of absurd pattern not empty"
-}

          ConP co n pl -> do
                 boolToError ("can only match mixed-variant arguments") $
                   leqPol (polarity dec) Mixed
                 when (isFunType av) $ fail ("higher-order matching of pattern " ++ show p ++ " at type " ++ show av ++ " not allowed")     
-- TODO: ensure that matchings against erased arguments are forced 
--                 when (erased dec) $ throwErrorMsg $ "checkPattern: cannot match on erased argument " ++ show p ++ " : " ++ show av 
-- WRONG:                 let flex1 = if erased' then MaxMatches 1 av : flex else flex
                 let flex1 = flex
                 when erased' $ do
                   mcenvs <- matchingConstructors av
                   case mcenvs of
                      Nothing -> recoverFail $ "checkPattern: not a data type: " ++ show av
                      Just [] -> recoverFail $ "checkPattern: no constructor matches erased type " ++ show av 
                      Just [(ci, _)] | cName ci == n -> return ()
                      _ -> recoverFail $ "checkPattern: constructor " ++ n ++ " of erased argument " ++ show p ++ " : " ++ show av ++ " not forced"

                 (ConSig nPars sz recOccs vc) <- lookupSymb n 
                 when (co == CoInd && sz == Sized && not (isDotPattern (pl !! nPars))) $
                   fail $ "in pattern " ++ show p  ++ ", coinductive size sub pattern " ++ show (pl !! nPars) ++ " must be dotted"
                 (flex',ins',cxt',vc',ple,absp) <- checkPatterns erased' flex1 ins vc pl
                 when (isFunType vc') $ fail ("higher-order matching of pattern " ++ show p ++ " of type " ++ show vc' ++ " not allowed")  
                 let flexgen = concat $ map (\ g -> case g of
                        DotFlex i _ _ -> [i]
                        MaxMatches{} -> []) flex'
                     -- fst $ unzip flex'
--                  av1 <- sing (environ cxt') (patternToExpr p) vc' 
--                  av2 <- sing (environ cxt') (patternToExpr p) av 
--                  subst <- local (\ _ -> cxt') $ inst flexgen VSet av1 av2
                 
                 -- need to evaluate the erased pattern!
                 let pe = ConP co n ple -- erased pattern
                 let epe = patternToExpr pe
                 pv0 <- local (\ _ -> cxt') $ whnf' epe
--                 pv0 <- patternToVal p -- THIS USE should be ok, since the new GENs are not in the global context yet, only in cxt' -- NO LONGER ok with erasure!
                 -- traceM $ "sucessfully computed value " ++ show pv0 ++ " of pattern " ++ show epe
                 subst <- local (\ _ -> cxt') $ do
                   case av of  -- TODO: need subtyping-unify instead of unify??
                     VSing vav av0 -> do
                       vav <- whnfClos vav
                       inst Pos flexgen av0 pv0 vav
                     _ -> unifyIndices flexgen vc' av  -- vc' <= av ?!
                   -- THIS IMPLEMENTATION RELIES HEAVILY ON INJECTIVITY OF DATAS

{- moved to checkRHS
                 -- apply substitution to measures in environment
                 let mmu = (envBound . environ) cxt'
                 mmu' <- Traversable.mapM (substitute subst) mmu    
-}
{-
                 ins'' <- compSubst ins' subst
                 vb <- substitute ins'' vb
                 delta' <- substitute ins'' delta'
-}
                 ins''   <- compSubst ins' subst -- 2010-07-27 not ok to switch!
                 delta'' <- substitute ins'' (context cxt')
                 traceCheckM $ "delta'' = " ++ show delta''
                 pv  <- up False pv0 av
                 vb  <- whnf (update env x pv) b
                 vb' <- substitute ins'' vb  -- ?? why not first subst and then whnf?
                 -- traceCheck ("Returning type " ++ show vb') $
                 return (flex', ins'', cxt' { context = delta'' }, vb',
                         maybeErase pe, absp)
{- DO NOT UPDATE measure here, its done in checkRHS
                 return (flex', ins'', cxt' { context = delta'', environ = (environ cxt') { envBound = mmu' } }, vb',
                         maybeErase pe, absp)
-}
          SuccP p2 -> do  
                 when (av /= VSort Size) (throwErrorMsg "checkPattern: expected type Size") 

                 when (isSuccessorPattern p2) $
                   fail ("cannot match against deep successor pattern " 
                    ++ show p ++ "; type: " ++ show v)

                 co <- asks mutualCo
                 when (co /= CoInd) $
                   fail ("successor pattern only allowed in cofun")

                 enter ("checkPattern " ++ show p ++" : matching on size, checking that target " ++ show v ++ " ends in correct coinductive sized type") $
                   newWithGen x (Domain av decEr) $ \ i xv -> do
                     vb0 <- whnf (update env x xv) b
                     endsInSizedCo i vb0
{-
                 when erased' $ throwErrorMsg $ "checkPattern: cannot match on erased argument " ++ show p ++ " : " ++ show av 
-}
                 cxt <- ask
                 (flex',ins',cxt',v',p2e,absp) <- checkPattern erased' flex ins (VSort Size `arrow` VSort Size) p2
                 -- leqVal Mixed delta' VSet VSize av -- av = VSize 
                 let pe = SuccP p2e
                 pv0 <- local (\ _ -> cxt') $ whnf' $ patternToExpr pe
                 -- pv0 <- patternToVal p -- RETIRE patternToVal
                 pv  <- up False pv0 av
                 vb  <- whnf (update env x pv) b
{-
                 endsInCoind <- endsInSizedCo pv vb
                 when (not endsInCoind) $ throwErrorMsg $ "checkPattern " ++ show p ++" : cannot match on size since target " ++ show v ++ " does not end in correct coinductive sized type"
-}
                 return (flex',ins',cxt',vb,pe,absp) -- note that erased'==False

          DotP e -> newWithGen (show p) (Domain av decEr) $ \ k v -> do
                       vb <- whnf (update env x v) b
                       cxt' <- ask
                       -- traceCheck ("Returning type " ++ show vb) $
                       return (DotFlex k e (Domain av decEr) : flex
                              ,ins
                              ,cxt'
                              ,vb                         -- e leads to unbound ids in evaluating patterns!
                              ,maybeErase $ DotP $ Meta k -- e -- Meta k
                              ,False) -- TODO: Erase in e/ Meta subst!
{- original code 
                    do let (k, delta') = cxtPush dec av delta
                       vb <- whnf (update env x (VGen k)) b
                       return ((k,(e,Domain av dec)):flex
                              ,ins
                              ,rho
                              ,delta'
                              ,vb)
-}
    _ -> throwErrorMsg $ "checkpattern " ++ show v
  ) `throwTrace` ("pattern " ++ show p)
 

{- New treatment of size matching  (see examples/Sized/Cody.ma)

sized data O : Size -> Set
{ Z : [i : Size] -> O ($ i)
; S : [i : Size] -> O i -> O ($ i)
; L : [i : Size] -> (Nat -> O i) -> O ($ i)
; M : [i : Size] -> O i -> O i -> O ($ i)
}

fun deep : [i : Size] -> O i -> Nat -> Nat
{ deep i4 (M i3 (L j2 f) (S i2 (S i1 (S i x)))) n
  = deep _ (M _ (L _ (pre _ f)) (S _ (f n))) (succ (succ (succ n)))
; deep i x n = n   
}

Explicit form:  Size variables and their constraints are noted explicitely,
to be able to do untyped call extraction in the termination module.

 deep i4 
  (M (i4 > i3) 
       (L (i3 > j2) f) 
       (S (i3 > i2) 
            (S (i2 > i1) 
                 (S (i1 > i) x)))) n
  = deep _ (M _ (L _ (pre _ f)) (S _ (f n))) (succ (succ (succ n)))

i4, i3, ... are all rigid variables with constraints between them.
There is a tree hierarchy, but I do not know whether I can exploit
this.

  i4 > i3 > i2 > i1 > i
          > j3

This could be stored in a union-find-like data structure, or just in
the constraint matrix.

How to pattern match?

  id : [i : Size] -> List i -> List i
  id i (cons (i > j) x xs) = cons j x (id j xs)

Only a size variable matches a size arguments

  match  (cons (i > j) x xs)   against   List i
  get    cons : [j : Size] -> Nat -> List j -> List ($ j)
  yield  x : Nat, xs : List j, cons j x xs : List ($ j)
  check  List ($ j) <= List i 
 -}

{- RETIRED
-- checkDot does not need to extract
checkDot :: Substitution -> DotFlex -> TypeCheck ()
checkDot subst (i,(e,it)) = enter ("dot pattern " ++ show e) $ 
  case (lookup i subst) of
    Nothing -> fail $ "not instantiated"
    Just v -> do 
      tv <- substitute subst (typ it)
      ask >>= \ ce -> traceCheckM ("checking dot pattern " ++ show ce ++ " |- " ++ show e ++ " : " ++ show (decor it) ++ " " ++ show tv) 
      applyDec (decor it) $ do
        checkExpr e tv
        v' <-  whnf' e -- TODO: has subst erased terms?
        enter ("inferred value " ++ show v ++ " does not match given dot pattern value " ++ show v') $
          eqVal Pos tv v v'  
-}

-- checkDot does not need to extract
checkGoal :: Substitution -> Goal -> TypeCheck ()
checkGoal subst (DotFlex i e it) = enter ("dot pattern " ++ show e) $ 
  case (lookup i subst) of
    Nothing -> fail $ "not instantiated"
    Just v -> do 
      tv <- substitute subst (typ it)
      ask >>= \ ce -> traceCheckM ("checking dot pattern " ++ show ce ++ " |- " ++ show e ++ " : " ++ show (decor it) ++ " " ++ show tv) 
      applyDec (decor it) $ do
        checkExpr e tv
        v' <-  whnf' e -- TODO: has subst erased terms?
        enter ("inferred value " ++ show v ++ " does not match given dot pattern value " ++ show v') $
          leqVal Pos tv v v' -- WAS: eqVal 
checkGoal subst (MaxMatches n av) = do
  av' <- substitute subst av
  mcenvs <- matchingConstructors av'
  maybe (recoverFail $ "not a data type: " ++ show av')
   (\ cenvs -> 
      if length cenvs > n then recoverFail $
        if n==0 then "absurd pattern does not match since type " ++ show av' ++ " is not empty"
         else -- n == 1 and 
           "more than one constructor matches type " ++ show av'
       else return ())
   mcenvs  


checkRHS :: Substitution -> Expr -> TVal -> TypeCheck Extr
checkRHS ins rhs v = do
   traceCheckM ("checking rhs " ++ show rhs ++ " : " ++ show v)  
   enter "right hand side" $ do
     -- first updated measure according to substitution for dot variables
     cxt <- ask
     let rho = environ cxt
     mmu' <- Traversable.mapM (substitute ins) (envBound rho)
     local (\ _ -> cxt { environ = rho { envBound = mmu' }}) $
       activateFuns $ 
         checkExpr rhs v  


  
-- TODO type directed unification

-- unifyIndices flex tv1 tv2
-- tv1 = D pars  inds  is the type of the pattern
-- tv2 = D pars' inds' is the type matched against
-- Note that in this case we can unify without using the principle of
-- injective data type constructors, 
-- we are only calling unifyIndices from the constructor pattern case 
-- in Checkpattern
unifyIndices :: [Int] -> Val -> Val -> TypeCheck Substitution
unifyIndices flex v1 v2 = ask >>= \ cxt -> enter ("unifyIndices " ++ show (context cxt) ++ " |- " ++ show v1 ++ " ?<=" ++ show Pos ++ " " ++ show v2) $ do 
-- {-
  case (v1,v2) of
    (VApp (VDef (DefId Dat d1)) vl1, VApp (VDef (DefId Dat d2)) vl2) | d1 == d2 -> do
      (DataSig { numPars = np, symbTyp = tv, positivity = posl}) <- lookupSymb d1
      instList posl flex tv vl1 vl2 -- unify also parameters to solve dot patterns
    _ -> 
-- -}
         inst Pos flex (VSort Type) v1 v2 
-- throwErrorMsg ("unifyIndices " ++ show v1 ++ " =?= " ++ show v2 ++ " failed, not applied to data types")

-- unify, but first produce whnf
instWh :: Pos -> [Int] -> TVal -> Val -> Val -> TypeCheck Substitution
instWh pos flex tv w1 w2 = do
  v1 <- whnfClos w1
  v2 <- whnfClos w2
  inst pos flex tv v1 v2

-- match v1 against v2 by unification , yielding a substition
inst :: Pos -> [Int] -> TVal -> Val -> Val -> TypeCheck Substitution
inst pos flex tv v1 v2 = ask >>= \ cxt -> enter ("inst " ++ show (context cxt) ++ " |- " ++ show v1 ++ " ?<=" ++ show pos ++ " " ++ show v2 ++ " : " ++ show tv) $ do 
--  case tv of
--    (VPi dec x av env b) -> 
  case (v1,v2) of
    (VGen k,_) | elem k flex -> do
                       l <- getLen
                       noc <- nocc l v1 v2 
                       case noc of 
                         True -> return [(k,v2)]
                         False -> throwErrorMsg "occurs check failed"
    (_,VGen k) | elem k flex -> do
                   l <- getLen
                   noc <- nocc l v2 v1 
                   case noc of 
                         True -> return [(k,v1)]
                         False -> throwErrorMsg "occurs check failed"
{- MOVED to unifyIndices
    -- injectivity of data type constructors is unsound in general
    (VApp (VDef (DefId Dat d1)) vl1, VApp (VDef (DefId Dat d2)) vl2) | d1 == d2 ->  do
         sig <- gets signature 
         instList flex (symbTyp (lookupSig d1 sig)) vl1 vl2
-}

    (VApp (VDef (DefId Dat d1)) vl1, VApp (VDef (DefId Dat d2)) vl2) | d1 == d2 ->  do
         (DataSig { numPars = np, symbTyp = tv, positivity = posl }) <- lookupSymb d1 
         instList' np posl flex tv vl1 vl2 -- ignore parameters (first np args)
           -- this is sound because we have irrelevance for parameters
           -- we assume injectivity for indices


    (VApp (VDef (DefId (Con _) c1)) vl1,VApp (VDef (DefId (Con _) c2)) vl2) | c1 == c2 -> do
         sige <- lookupSymb c1
         instList [] flex (symbTyp sige) vl1 vl2
    (VSucc v1',VSucc v2') -> instWh pos flex tv v1' v2'
    (VSucc v, VInfty) -> instWh pos flex tv v VInfty
    (VSing v1' tv1, VSing v2' tv2) -> do 
      subst <- inst pos flex tv tv1 tv2
      u1 <- substitute subst v1'
      u2 <- substitute subst v2'
      tv1' <- substitute subst tv1
      inst pos flex tv1' u1 u2 >>= compSubst subst
      
-- HACK AHEAD
    (VUp v1 _, _) -> inst pos flex tv v1 v2
    (_, VUp v2 _) -> inst pos flex tv v1 v2
--    (VUp v1' a1, VUp v2' a2) -> instList flex [a1,v1'] [a2,v2']
--     (VPi dec x1 av1 env1 b1, VPi dec x2 av2 env2 b2) ->
      
{- TODO: REPAIR THIS
    _ -> traceCheck ("inst: WARNING! assuming " ++ show (context cxt) ++ " |- " ++ show v1 ++ " == " ++ show v2) $
           return [] -- throwErrorMsg $ "inst: NYI"
 -}   
    _ -> do leqVal pos tv v1 v2 `throwTrace` ("inst: leqVal " ++ show v1 ++ " ?<=" ++ show pos ++ " " ++ show v2 ++ " : " ++ show tv ++ " failed")
            return []

instList :: [Pos] -> [Int] -> TVal -> [Val] -> [Val] -> TypeCheck Substitution
instList posl flex tv vl1 vl2 = do
  -- vl1 <- mapM whnfClos vl1
  -- vl2 <- mapM whnfClos vl2
  instList' 0 posl flex tv vl1 vl2

-- unify lists, ignoring the first np items
instList' :: Int -> [Pos] -> [Int] -> TVal -> [Val] -> [Val] -> TypeCheck Substitution
instList' np posl flex tv [] [] = return []
instList' np posl flex tv (v1:vl1) (v2:vl2) = do
  v1 <- whnfClos v1
  v2 <- whnfClos v2
  if (np <= 0 || isMeta flex v1 || isMeta flex v2) then 
    case tv of
      (VPi dec x av env b) -> do    
        subst <- inst (headPosl posl) flex av v1 v2
        vl1' <- mapM (substitute subst) vl1
        vl2' <- mapM (substitute subst) vl2
        v    <- substitute subst v1
        env' <- substitute subst env
        vb   <- whnf (update env' x v) b
        subst' <- instList' (np - 1) (tailPosl posl) flex vb vl1' vl2'
        compSubst subst subst'
   else
    case tv of
      (VPi dec x av env b) -> do    
        vb   <- whnf (update env x v2) b
        instList' (np - 1) (tailPosl posl) flex vb vl1 vl2
instList' np pos flex tv vl1 vl2 = fail $ "internal error: instList' " ++ show (np,pos,flex,tv,vl1,vl2) ++ " not handled" 

headPosl :: [Pos] -> Pos      
headPosl [] = Mixed
headPosl (pos:_) = pos

tailPosl :: [Pos] -> [Pos]      
tailPosl [] = []
tailPosl (_:posl) = posl


isMeta :: [Int] -> Val -> Bool
isMeta flex (VGen k) = k `elem` flex
isMeta _ _ = False

-- overload `subst`
class Substitute a where
  substitute :: Substitution -> a -> TypeCheck a

instance Substitute (Measure Val) where
  substitute subst (Measure mu) = do
    mu' <- mapM (substitute subst) mu
    return $ Measure mu'

instance Substitute (Bound Val) where
  substitute subst (Bound mu1 mu2) = do
    mu1' <- substitute subst mu1
    mu2' <- substitute subst mu2
    return $ Bound mu1' mu2'

-- substitute generic variable in value
instance Substitute Val where
  substitute subst v =  
    case v of
      VGen k -> case lookup k subst of
                  Nothing ->  return $ v
                  Just v' ->  return $ v'
      VApp v1 vl -> do v1' <- substitute subst v1
                       vl' <- mapM (substitute subst) vl
                       foldM app v1' vl'  -- does not do anything for empty list vl'
      VSing v1 vt -> do v1' <- substitute subst v1
                        vt' <- substitute subst vt
                        return $ vSing v1' vt'  -- TODO: Check reevaluation necessary? 
      VSucc v1 -> do v1' <- substitute subst v1
                     return $ succSize v1'
      VMax vs  -> do vs' <- mapM (substitute subst) vs
                     return $ maxSize vs'
      VCase v1 env cl -> do v1' <- substitute subst v1
                            env' <- substitute subst env
                            return $ VCase v1' env' cl
      VMeasured mu bv -> do
          mu' <- substitute subst mu
          bv'   <- substitute subst bv
          return $ VMeasured mu' bv'
      VGuard beta bv -> do
          beta' <- substitute subst beta
          bv'   <- substitute subst bv
          return $ VGuard beta' bv'
      VPi dec x av env b ->  
          do av'  <- substitute subst av
             env' <- substitute subst env
             return $ VPi dec x av' env' b
      VLam x env b -> do env' <- substitute subst env
                         return $ VLam x env' b
      VClos env e  -> do env' <- substitute subst env
                         return $ VClos env' e
      VUp v1 vt -> do v1' <- substitute subst v1
                      vt' <- substitute subst vt
                      up False v1' vt'
      VSort s   -> substitute subst s >>= return . VSort
      VZero     -> return $ v
      VInfty    -> return $ v
      VIrr      -> return $ v 
      VDef id   -> return $ vDef id  -- because empty list of apps will be rem.
--      VCon co n -> return $ v
      VMeta x n -> return $ v

instance Substitute (Sort Val) where
  substitute subst s =
    case s of
      Set v -> do v' <- substitute subst v
                  return $ Set v'
      CoSet v -> substitute subst v >>= return . CoSet
      Tm    -> return Tm
      Type  -> return Type
      Size  -> return Size
      TSize -> return TSize

instance Substitute SemCxt where
  substitute subst delta = do  
    cxt' <- mapMapM (Traversable.mapM (substitute subst)) (cxt delta)
    return $ delta { cxt = cxt' }
{-
instance Substitute a => Substitute (Maybe a) where
  substitute subst Nothing  = return $ Nothing
  substitute subst (Just a) = substitute subst a >>= return . Just 
-}

-- substitute in environment
instance Substitute Env where
  substitute subst (Environ rho mmeas) = do
    rho' <- mapM (\(x,v) -> substitute subst v >>= return . (x,)) rho
    mmeas' <- Traversable.mapM (substitute subst) mmeas
    return $ Environ rho' mmeas'
{-
substitute subst [] = return $ []
substitute subst ((x,v):env) = do  v'   <- substitute subst v
                                 env' <- substitute subst env
                                 return $  (x,v'):env'
-}


-- "merge" substitutions by first applying the second to the first, then 
-- appending them
compSubst :: Substitution -> Substitution -> TypeCheck Substitution
compSubst subst1 subst2 = do
    subst1' <- mapM (\ (x,v) -> substitute subst2 v >>= return . (x,)) subst1
    return $ subst1' ++ subst2               
{-
compSubst subst1 subst2 = do
    let (dom1,tg1) = unzip subst1
    tg1' <- mapM (substitute subst2) tg1
    let subst1' = zip dom1 tg1'
    return $ subst1' ++ subst2               
-}

---- Size checking
--------------

-- check wether the data type is sized type


-- check data declaration type 
-- called from typeCheckDeclaration (DataDecl{})
-- parameters : number of params, type 
szType :: Co -> Int -> TVal -> TypeCheck ()
szType co p tv = doVParams p tv $ \ tv' -> do
    let polsz = if co==Ind then Pos else Neg
    case tv' of
      VPi dec x (VSort Size) env b | not (erased dec) && polarity dec == polsz -> return ()
      _ -> throwErrorMsg $ "not a sized type, target " ++ show tv' ++ " must have non-erased domain " ++ show (VSort Size) ++ " with polarity " ++ show polsz
                 
-- check data constructors
-- called from typeCheckConstructor
szConstructor :: Name -> Co -> Int -> TVal -> TypeCheck ()
szConstructor n co p tv = enter ("szConstructor " ++ show n ++ " : " ++ show tv) $ do 
  doVParams p tv $ \ tv' -> 
    case tv' of
       VPi dec x av@(VSort Size) env b -> 
          newWithGen x (Domain av dec) $ \ k xv -> do
            bv <- whnf (update env x xv) b 
            szSizeVarUsage n co p k bv
       _ -> fail $ "not a valid sized constructor: expected size quantification"

szSizeVarUsage :: Name -> Co -> Int -> Int -> TVal -> TypeCheck ()
szSizeVarUsage n co p i tv = enter ("szSizeVarUsage of " ++ show i ++ " in " ++ show tv) $ 
    case tv of
       VPi dec x av env b -> do 
          szSizeVarDataArgs n p i av  -- recursive calls of for D..i..
          enter ("checking " ++ show av ++ " to be " ++
              (if co == CoInd then "antitone" else "isotone") ++ " in variable "
              ++ show i) $
            szMono co i av                -- monotone in i
          new x (Domain av dec) $ \ xv -> do
            bv <- whnf (update env x xv) b
            szSizeVarUsage n co p i bv
                
       _ -> szSizeVarTarget p i tv

-- check that Target is of form D ... (Succ i) ... 
szSizeVarTarget :: Int -> Int -> TVal -> TypeCheck ()
szSizeVarTarget p i tv = enter ("szSizeVarTarget, variable " ++ show i ++ " argument no. " ++ show p ++ " in " ++ show tv) $ do
    let err = "expected target " ++ show tv ++ " of size " ++ show (VSucc (VGen i)) 
    case tv of
       (VApp d vl) -> do
               v0 <- whnfClos (vl !! p)
               case v0 of
                 (VSucc (VGen i')) | i == i' -> return ()
                 _ -> fail err
       _ -> fail err


-- check that rec. arguments are of form D ... i ....
-- and size used nowhere else ?? -- Andreas, 2009-11-27 TOO STRICT!
{- accepts, for instance

   Nat -> Ord i      as argument of a constructor of  Ord ($ i)
   List (Rose A i)   as argument of a constructor of  Rose A ($i)
 -}
szSizeVarDataArgs :: Name -> Int -> Int -> TVal -> TypeCheck ()
szSizeVarDataArgs n p i tv = enter ("sizeVarDataArgs " ++ show i ++ " in " ++ show tv) $ do
   case tv of
     
     {- case D pars sizeArg args -} 
     VApp (VDef (DefId Dat m)) vl | n == m -> do 
        v0 <- whnfClos $ vl !! p
        case v0 of
          (VGen i') | i' == i  -> do 
              let rargs = take p vl ++ drop (p+1) vl
              k <- getLen
              mapM_ (\ v -> (nocc k (VGen i) v) >>= boolToError ("variable " ++ show i ++ " may not occur in " ++ show v)) rargs
          _ -> fail $ "wrong size index " ++ show v0 ++ " at recursive occurrence " ++ show tv

-- not necessary: check for monotonicity above
--     {- case D' pars sizeArg args -}
--     VApp (VDef m) vl | n /= m -> do 

     VApp v1 vl -> mapM_ (\ v -> whnfClos v >>= szSizeVarDataArgs n p i) (v1:vl)

     VPi dec x av env b -> do 
       szSizeVarDataArgs n p i av
       new x (Domain av dec) $ \ xv -> do
          bv <- whnf (update env x xv) b
          szSizeVarDataArgs n p i bv
              
     VLam x env b -> 
       addName x $ \ xv -> do
         bv <- whnf (update env x xv) b
         szSizeVarDataArgs n p i bv           
     _ -> return ()

{- REMOVED, 2009-11-28, replaced by monotonicity check
     VGen i' -> return $ i' /= i
     VSucc tv' -> szSizeVarDataArgs n p i tv'
 -}

-- doVParams number_of_params constructor_or_datatype_signature
-- skip over parameters of type signature of a constructor/data type
doVParams :: Int -> TVal -> (TVal -> TypeCheck a) -> TypeCheck a
doVParams 0 tv k = k tv
doVParams p (VPi dec x av env b) k = 
  new x (Domain av dec) $ \ xv -> do 
    bv <- whnf (update env x xv) b
    doVParams (p - 1) bv k

--------------------------------------
-- check for admissible  type

{- 

 - admissibility needs to be check clausewise, because of Karl's example

   fun nonAdmissibleType : Unit -> Set

   fun diverge : (u : Unit) -> nonAdmissibleType u
   {
     diverge unit patterns = badRhs
   }
   
 - the type must be admissible in the current position 
   only if the size pattern is a successor.  
   If the pattern is a variable, then there is no induction on that size
   argument, so no limit case, so no upper semi-continuity necessary
   for the type.

 - when checking

     ... (s i) ps  admissible  (j : Size) -> A

   we will check
  
     A  admissible in j

   and continue with

     ... ps  admissible  A[s i / j]

   just to maintain type wellformedness.  The (s i) in A does not
   really matter, since there is no case distinction on ordinals.

 - a size pattern which is not inductive (meaning there is an
    inductive type indexed by that size) nor coinductive (meaning that
    the result type is coinductive and is indexed by that size) must
    be flagged unusable for termination checking.  

 - the fun/cofun distinction could be inferred by the termination checker
   or be clausewise as in Agda 2

-}


admFunDef :: Co -> [Clause] -> TVal -> TypeCheck [Clause]
admFunDef co cls tv = do
  (cls, inco) <- admClauses cls tv
  when (co==CoInd && not (co `elem` inco)) $ 
    fail $ show tv ++ " is not a type of a cofun" -- ++ if co==Ind then "fun" else "cofun"
  return cls 

admClauses :: [Clause] -> TVal -> TypeCheck ([Clause], [Co])
admClauses [] tv = return ([], [])
admClauses (cl:cls) tv = do
  (cl',inco) <- admClause cl tv
  (cls',inco') <- admClauses cls tv
  return (cl' : cls', inco ++ inco')

admClause :: Clause -> TVal -> TypeCheck (Clause, [Co])
admClause (Clause ps e) tv = -- traceCheck ("admClause: admissibility of patterns " ++ show ps) $
   introPatVars ps $ do 
     vs <- mapM (whnf' . patternToExpr) ps
     let pvs = zip ps vs
     introPatTypes pvs tv $ \ _ -> do
       (ps', inco) <- admPatterns pvs tv
       return (Clause ps' e, inco)

admPatterns :: [(Pattern,Val)] -> TVal -> TypeCheck ([Pattern], [Co])
admPatterns [] tv = do
  isCo <- endsInCo tv
  return ([], if isCo then [CoInd] else [])
admPatterns ((p,v):pvs) tv = do
   (p, inco1)  <- admPattern p tv
   bv <- piApp tv v
   (ps, inco2) <- admPatterns pvs bv
   return (p:ps, inco1 ++ inco2)



-- introduce variables bound in pattern into the environment
-- extend delta by generic values but do not introduce their types
introPatVar :: Pattern -> TypeCheck a -> TypeCheck a
introPatVar p cont = 
    case p of
      VarP n -> addName n $ \ _ -> cont 
      SizeP m n -> addName n $ \ _ -> cont 
      ConP co n pl -> introPatVars pl cont
      SuccP p -> introPatVar p cont
      DotP e -> cont
      AbsurdP -> cont

introPatVars :: [Pattern] -> TypeCheck a -> TypeCheck a
introPatVars [] cont = cont
introPatVars (p:ps) cont = introPatVar p $ introPatVars ps $ cont

-- if the bindings name->gen are already in the environment
-- we can now bind the gen to their types
introPatType :: (Pattern,Val) -> TVal -> (TVal -> TypeCheck a) -> TypeCheck a
introPatType (p,v) tv f = do
  v <- whnfClos v
  case tv of
    VPi dec x av env b -> do
       bv <- whnf (update env x v) b
       matchPatType (p,v) av $ f bv

introPatTypes :: [(Pattern,Val)] -> TVal -> (TVal -> TypeCheck a) -> TypeCheck a
introPatTypes pvs tv f = do
  case pvs of
    [] -> f tv
    (pv:pvs') -> introPatType pv tv $ \ tv' -> introPatTypes pvs' tv' f

matchPatType :: (Pattern, Val) -> TVal -> TypeCheck a -> TypeCheck a
matchPatType (p,v) av cont = 
       case (p,v) of
                                                   -- erasure does not matter!
          (VarP y, VGen k) -> setType k (Domain av defaultDec) $ cont

          (SizeP z y, VGen k) -> setType k (Domain av defaultDec) $ cont

          (ConP co n [], _) -> cont

          (ConP co n pl, VApp (VDef (DefId Con{} _)) vl) -> do
             sige <- lookupSymb n
             let vc = symbTyp sige
             introPatTypes (zip pl vl) vc $ \ _ -> cont
 
          (SuccP p2, VSucc v2) -> matchPatType (p2, v2) (VSort Size) $ cont 

          (DotP e, _) -> cont
          (AbsurdP, _) -> cont
          
          _ -> fail $ "matchPatType: IMPOSSIBLE " ++ show (p,v)

{-
-- turn a pattern into a value
-- extend delta by generic values but do not introduce their types
evalPat :: Pattern -> (Val -> TypeCheck a) -> TypeCheck a
evalPat p f = 
    case p of
      VarP n -> addName n f 
      ConP co n [] -> f (VCon co n)
      ConP co n pl -> evalPats pl $ \ vl -> f (VApp (VCon co n) vl)
      SuccP p -> evalPat p $ \ v -> f (VSucc v)
-- DOES NOT WORK SINCE e has unbound variables
      DotP e -> do
        v <- whnf' e
        f v

evalPats :: [Pattern] -> ([Val] -> TypeCheck a) -> TypeCheck a
evalPats [] f = f []
evalPats (p:ps) f = evalPat p $ \ v -> evalPats ps $ \ vs -> f (v:vs) 
-}

{-
evalPat :: Pattern -> TypeCheck (State TCContext Val)
evalPat p = 
    case p of
      VarP n -> return $ State $ \ ce ->
        let (k, delta) = cxtPushGen (context ce)
            rho = update n (VGen k) (environ ce)
        in  (VGen k, TCContext { context = delta, environ = rho })
      ConP co n [] -> return (VCon co n)
      ConP co n pl -> do
        vl <- mapM evalPat pl 
        return (VApp (VCon co n) vl)
      SuccP p -> do
       v <- evalPat p
       return (VSucc v)   
-- TODO: does not work!
--      DotP e -> return $ State $ \ ce ->
-}     
         

-- | endsInSizedCo i tv = True  if  tv = Gamma -> D pars i indexes
endsInSizedCo :: Int -> TVal -> TypeCheck ()
endsInSizedCo i tv  = enter ("endsInSizedCo: " ++ show tv) $ do
   let fail1 = fail $ "endsInSizedCo: target " ++ show tv ++ " of corecursive function is neither a CoSet or codata of size " ++ show (VGen i) ++ " nor a tuple type" 
   case tv of
      VSort (CoSet (VGen i)) -> return ()  
      VMeasured mu bv -> endsInSizedCo i bv
      VPi dec x av env b -> new x (Domain av dec) $ \ gen -> do
         isInd <- szUsed Ind i av

         when (not isInd) $
           szAntitone i av `newErrorMsg` 
            ("type " ++ show av ++ " not lower semi continuous in v" ++ show i)

         bv <- whnf (update env x gen) b
         endsInSizedCo i bv
      VApp (VDef (DefId Dat n)) vl -> do 
         sige <- lookupSymb n 
         case sige of
            DataSig { numPars = np, isSized = Sized, isCo = CoInd } 
              | length vl > np -> do
                 v <- whnfClos $ vl !! np 
                 if v == (VGen i) then return () else fail1
-- we also allow the target to be a tuple if all of its components 
-- fulfill "endsInSizedCo"
            DataSig { symbTyp = dv, numPars = np, isSized = sz, 
                      constructors = [ci], isTuple = True } -> do
              -- match target of constructor against tv to instantiate
              let (_, ps) = cPatFam ci 
              --  c : ... -> D ps
              mrho <- nonLinMatchList False emptyEnv ps vl dv
              case mrho of
                Nothing -> fail $ "endsInSizedCo: panic: target type " ++ show tv ++ " is not an instance of the only constructor "
                Just rho -> enter ("endsInSizedCo: detected tuple target, checking components") $ 
                  fieldsEndInSizedCo i (cFields ci) rho 
            _ -> fail1
      _ -> fail $ "endsInSizedCo: target " ++ show tv ++ " of corecursive function is neither a function type nor a codata nor a tuple type"


fieldsEndInSizedCo :: Int -> [FieldInfo] -> Env -> TypeCheck ()
fieldsEndInSizedCo i fis rho0 = enter ("fieldsEndInSizedCo: checking fields of tuple type " ++ show fis ++ " in environment " ++ show rho0) $
  loop fis rho0 where  
    loop [] rho = return ()
    -- nothing to check for erased index fields
    loop (f : fs) rho | fClass f == Index && erased (fDec f) = 
      loop fs rho
    loop (f : fs) rho | fClass f == Index = do
      tv <- whnf rho (fType f) 
      endsInSizedCo i tv
      loop fs rho
    loop (f : fs) rho = do
      tv <- whnf rho (fType f) 
      when (not $ erased (fDec f)) $ endsInSizedCo i tv
      -- for non-index fields, value is not given by matching, so introduce
      -- generic value
      new (fName f) (Domain tv (fDec f)) $ \ xv -> do
        let rho' = update rho (fName f) xv
        -- do not need to check erased fields?
        loop fs rho'



{-
-- | endsInSizedCo v tv = True  if  tv = Gamma -> D pars v' indexes
-- for v = v' : Size
endsInSizedCo :: Val -> TVal -> TypeCheck Bool
endsInSizedCo v tv  = -- traceCheck ("endsInCo: " ++ show tv) $
   case tv of
      VPi dec x av env b -> new x (Domain av dec) $ \ gen -> do
         bv <- whnf (update env x gen) b
         endsInSizedCo v bv
      VApp (VDef (DefId Dat n)) vl -> do 
         sig <- gets signature
         case (lookupSig n sig) of
            DataSig { numPars = np, isSized = Sized, isCo = CoInd } 
              | length vl > np ->
                 eqValBool (VSort Size) (vl !! np) v
            _ -> return False
      _ -> return False
-}

endsInCo :: TVal -> TypeCheck Bool
endsInCo tv  = -- traceCheck ("endsInCo: " ++ show tv) $
   case tv of
      VPi dec x av env b -> new x (Domain av dec) $ \ gen -> do
         bv <- whnf (update env x gen) b
         endsInCo  bv

{-
      -- if not applied, it cannot be a sized type
      VDef (DefId Dat n) -> do 
         sig <- gets signature
         case (lookupSig n sig) of
            DataSig { isCo = CoInd } -> -- traceCheck ("found non-sized coinductive target") $ 
               return True
            _ -> return False
-}

      VApp (VDef (DefId Dat n)) vl -> do 
         sige <- lookupSymb n
         case sige of
            DataSig { isCo = CoInd } -> -- traceCheck ("found non-sized coinductive target") $ 
               return True
            _ -> return False
      _ -> return False

-- precondition: Pattern does not contain "Unusable"
admPattern :: Pattern -> TVal -> TypeCheck (Pattern, [Co])
admPattern p tv = -- trace ("admPattern " ++ show p ++ " type: " ++ show tv) $
  case tv of
      VPi dec x av env b -> newWithGen x (Domain av dec) $ \ k xv -> do
         bv <- whnf (update env x xv) b
  {-
         if p is successor pattern
         check that bv is admissible in k, returning subset of [Ind, CoInd]       
         p is usable if either CoInd or it is a var or dot pattern and Ind  
-}
         if isSuccessorPattern p then do
           inco <- admType k bv
           when (CoInd `elem` inco && not (shallowSuccP p)) $
             fail ("cannot match coinductive size against deep successor pattern " 
                    ++ show p ++ "; type: " ++ show tv)
           if (CoInd `elem` inco)
              || (inco /= [] && completeP p) 
            then return (p, inco)
            else return (UnusableP p, inco)
          else return (p, [])

      _ -> fail "admPattern: IMPOSSIBLE: pattern for a non-function type"

admType :: Int -> TVal -> TypeCheck [Co]
admType i tv = enter ("admType: checking " ++ show tv ++ " admissible in v" ++ show i) $
    case tv of 
       VPi dec x av env b -> do
          isInd <- szUsed Ind i av
          when (not isInd) $
            szAntitone i av `newErrorMsg` 
             ("type " ++ show av ++ " not lower semi continuous in v" ++ show i)  
          new x (Domain av dec) $ \ gen -> do
            bv <- whnf (update env x gen) b
            inco <- admType i bv
            if isInd then return (Ind : inco) else return inco
       _ -> do
          isCoind <- szUsed CoInd  i tv
          if isCoind then return [CoInd]
           else do
            szMonotone i tv
            return []         

szUsed :: Co -> Int -> TVal -> TypeCheck Bool
szUsed co i tv = -- traceCheck ("szUsed: " ++ show tv ++ " " ++ show co ++ " in v" ++ show i) $
    case tv of
         (VApp (VDef (DefId Dat n)) vl) -> 
             do sige <- lookupSymb n
                case sige of
                  DataSig { numPars = p 
                          , isSized = Sized 
                          , isCo = co' } | co == co' && length vl > p -> 
                      -- p is the number of parameters
                      -- it is also the index of the size argument
                      do s <- whnfClos $ vl !! p
                         case s of
                           VGen i' | i == i' -> return True
                           _ -> return False
                  _ -> return False
         _ -> return False











-- for inductive fun, and for every size argument i
-- - every argument needs to be either inductive or antitone in i 
-- - the result needs to be monotone in i 

{- szCheckIndFun admpos delta tv

 entry point for admissibility check for recursive functions
 - scans for the first size quantification 
 - passes on to szCheckIndFunSize
 - currently: also continues to look for the next size quantification...
 -}

szCheckIndFun :: [Int] -> TVal -> TypeCheck ()
szCheckIndFun admpos tv = -- traceCheck ("szCheckIndFun: " ++ show delta ++ " |- " ++ show tv ++ " adm?") $
      case tv of 
       VPi dec x av env b -> new x (Domain av dec) $ \ (VGen k) -> do
         bv <- whnf' b
         case av of
           VSort Size -> do 
             when (k `elem` admpos) $ 
               szCheckIndFunSize k bv
             szCheckIndFun admpos bv -- this is for lexicographic induction on sizes, I suppose?  Probably should me more fine grained!  Andreas, 2008-12-01
           _ -> szCheckIndFun admpos bv
       _ -> return ()


{- szCheckIndFunSize delta i tv

 checks whether type tv is admissible for recursion in index i
 - every argument needs to be either inductive or antitone in i 
 - the result needs to be monotone in i  
 -}

szCheckIndFunSize :: Int -> TVal -> TypeCheck ()
szCheckIndFunSize i tv = -- traceCheck ("szCheckIndFunSize: " ++ show delta ++ " |- " ++ show tv ++ " adm(v" ++ show i ++ ")?") $ 
    case tv of 
       VPi dec x av env b -> do
            szLowerSemiCont i av
            new' x (Domain av dec) $ do
              bv <- whnf' b
              szCheckIndFunSize i bv
       _ -> szMonotone i tv

{- szLowerSemiCont

 - check for lower semi-continuity [Abel, CSL 2006]
 - current approximation: inductive type or antitone
 -}
szLowerSemiCont :: Int -> TVal -> TypeCheck ()
szLowerSemiCont i av = -- traceCheck ("szlowerSemiCont: checking " ++ show av ++ " lower semi continuous in v" ++ show i) $
   ((szAntitone i av) `catchError` 
      (\ msg -> -- traceCheck (show msg) $
                   szInductive i av)) 
        `newErrorMsg` ("type " ++ show av ++ " not lower semi continuous in v" ++ show i)  
                                                     

{- checking cofun-types for admissibility

conditions:

1. type must end in coinductive type or in sized coinductive type 
   indexed by just a variable i which has been quantified in the type

2. in the second case, each argument must be inductive or antitone in i
   optimization:
     arguments types before the quantification over i can be ignored
-}

data CoFunType 
  = CoFun             -- yes, but not sized cotermination
  | SizedCoFun Int    -- yes an admissible sized type (the Int specifies the number of the recursive size argument)

{-
design:

admCoFun delta tv : IsCoFunType

   endsInCo delta tv (len delta) id

admEndsInCo delta tv firstVar jobs : IsCoFunType
   
   traverse tv, gather continutations in jobs, check for CoInd in the end
   
   if tv = (x:A) -> B
      push A on delta
      add the following task to jobs:
        check A for lower semicontinuity in delta
      continue on B

   if tv = Codata^i
      run (jobs i)
      if they return (), return YesSized Int, otherwise No

   if tv = Codata
      return Yes

   otherwise
      return No
 -}

-- {- TODO: FINISH THIS!!

admCoFun :: TVal -> TypeCheck CoFunType
admCoFun tv = do 
  l <- getLen 
  admEndsInCo tv l (\ i -> return ())

admEndsInCo :: TVal -> Int -> (Int -> TypeCheck ()) -> TypeCheck CoFunType
admEndsInCo tv firstVar jobs = -- traceCheck ("admEndsInCo: " ++ show tv) $
   case tv of
      VPi dec x av env b -> do
         l <- getLen
         let jobs' = (addJob l av jobs)
         new' x (Domain av dec) $ do
           bv <- whnf' b
           admEndsInCo bv firstVar jobs' 

{-
      -- if not applied, it cannot be a sized type
      VDef n -> do 
         sig <- gets signature
         case (lookupSig n sig) of
            DataSig { isCo = CoInd } -> -- traceCheck ("found non-sized coinductive target") $ 
               return CoFun
            _ -> throwErrorMsg $ "type of cofun does not end in coinductive type"
-}

      VApp (VDef (DefId Dat n)) vl -> do 
         sige <- lookupSymb n
         case sige of
            DataSig { isSized = NotSized, isCo = CoInd } -> -- traceCheck ("found non-sized coinductive target") $ 
               return CoFun
            DataSig { numPars = p, isSized = Sized, isCo = CoInd } | length vl > p -> -- traceCheck ("found sized coinductive target") $
              do 
               -- p is the number of parameters
               -- it is also the index of the size argument
               s <- whnfClos $ vl !! p
               case s of
                  VGen i -> do 
                     jobs i
                     return $ SizedCoFun $ i - firstVar 
                  _ -> throwErrorMsg $ "size argument in result type must be a variable" 
            _ -> throwErrorMsg $ "type of cofun does not end in coinductive type"

addJob :: Int -> TVal -> (Int -> TypeCheck ())
       -> (Int -> TypeCheck ())
addJob l tv jobs recVar = do
  -- is the "recursive" size variable actually in scope?
  jobs recVar
  when (recVar < l) $ szLowerSemiCont recVar tv

-- -}


{- szCheckCoFun  OBSOLETE!!

 entry point for admissibility check for corecursive functions
 - scans for the first size quantification 
 - passes on to szCheckIndFunSize
 - currently: also continues to look for the next size quantification
 - and checks in the end whether the target is a coinductive type


-- STALE COMMENT: for a cofun : arguments nocc i and result coinductive in i 
szCheckCoFun :: SemCxt -> TVal -> TypeCheck ()
szCheckCoFun delta tv = 
      case tv of
       VPi dec x av env b -> do
                let (k, delta') = cxtPush dec av delta 
                bv <- whnf (update env x (VGen k)) b
                case av of
                  VSize -> do szCheckCoFunSize delta' k bv
                              szCheckCoFun delta' bv
                  _ -> szCheckCoFun delta' bv 
       -- result 
       (VApp (VDef n) vl) -> 
          do sig <- gets signature
             case (lookupSig n sig) of
               (DataSig _ _ _ CoInd _) -> 
                   return ()
               _ -> throwErrorMsg $ "cofun doesn't target coinductive type"
       (VDef n)  -> 
          do sig <- gets signature
             case (lookupSig n sig) of
               (DataSig _ _ _ CoInd _) -> 
                   return ()
               _ -> throwErrorMsg $ "cofun doesn't target coinductive type"
       _ -> throwErrorMsg $ "cofun doesn't target coinductive type"

szCheckCoFunSize :: SemCxt -> Int -> TVal -> TypeCheck ()
szCheckCoFunSize delta i tv = -- traceCheck ("szco " ++ show tv) $
      case tv of 
       VPi dec x av env b ->  do  
             let (k, delta') = cxtPush dec av delta 
             bv <- whnf (update env x (VGen k)) b
             szLowerSemiCont delta i av
             szCheckCoFunSize delta' i bv
       -- result must be coinductive
       _ -> szCoInductive delta i tv

-}
              
{-
szMono :: Co -> Int -> TVal -> TypeCheck Bool
szMono co i tv = 
  (do case co of
         Ind   -> szMonotone i tv
         CoInd -> szAntitone i tv
      return True
  ) `catchError` (\ _ -> return False)
-}

szMono :: Co -> Int -> TVal -> TypeCheck ()
szMono co i tv = 
    case co of
         Ind   -> szMonotone i tv
         CoInd -> szAntitone i tv

szMonotone :: Int -> TVal -> TypeCheck ()
szMonotone i tv = traceCheck ("szMonotone: " -- ++ show delta ++ " |- " 
                              ++ show tv ++ " mon(v" ++ show i ++ ")?") $
 do
   let si = VSucc (VGen i)
   tv' <- substitute [(i,si)] tv
   leqVal Pos (VSort Type) tv tv'

szAntitone :: Int -> TVal -> TypeCheck ()
szAntitone i tv = traceCheck ("szAntitone: " -- ++ show delta ++ " |- " 
                              ++ show tv ++ " anti(v" ++ show i ++ ")?") $
 do
   let si = VSucc (VGen i)
   tv' <- substitute [(i,si)] tv
   leqVal Neg (VSort Type) tv tv'

-- checks if tv is a sized inductive type of size i 
szInductive :: Int -> TVal -> TypeCheck ()
szInductive i tv = szUsed' Ind i tv

-- checks if tv is a sized coinductive type of size i 
szCoInductive :: Int -> TVal -> TypeCheck ()
szCoInductive i tv = szUsed' CoInd i tv
            
szUsed' :: Co -> Int -> TVal -> TypeCheck ()
szUsed' co i tv =
    case tv of
         (VApp (VDef (DefId Dat n)) vl) -> 
             do sige <- lookupSymb n
                case sige of
                  DataSig { numPars = p, isSized = Sized, isCo =  co' } | co == co' && length vl > p -> 
                      -- p is the number of parameters
                      -- it is also the index of the size argument
                      do s <- whnfClos $ vl !! p
                         case s of
                           VGen i' | i == i' -> return ()
                           _ -> fail $ "expected size variable" 
                  _ -> fail $ "expected (co)inductive sized type"
         _ -> fail $ "expected (co)inductive sized type"

